/*******************************************************************************
This file is part of WhiteStork.

WhiteStork is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

WhiteStork is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License 
along with WhiteStork; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Copyright 2006 ComArch S.A.
*******************************************************************************/
#ifndef _DICTIONARY_ENGINE_BOOKMARK
#define _DICTIONARY_ENGINE_BOOKMARK

#ifdef __cplusplus
        extern "C" {
#endif

//______________________________________________________________________________
// *****************************************************************************
//************************************************************* HEADERS SECTION:
//------------------------------------------------------------------------------
// headers with unix types/functions - only for timers
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <fcntl.h>
#include <unistd.h>
//------------------------------------------------------------------------------
// header with GLIB definitions/functions/types
#include <glib.h>
//------------------------------------------------------------------------------
// header wit engine API
#include <dictionary_engine.h>
//------------------------------------------------------------------------------
// header wit sqlite 2.x API
#include <sqlite.h>
//------------------------------------------------------------------------------

//______________________________________________________________________________
// *****************************************************************************
//********************************************************* DEFINITIONS SECTION:
//------------------------------------------------------------------------------
// definitions for timer function - flag telling if we want to start or stop
// timing
#define TIMER_START     TRUE
#define TIMER_STOP      FALSE
//------------------------------------------------------------------------------
// definitions of version and format which engine handles
static const gchar* DIC_ENG_VERSION = "0.1"; 
static const gchar* DIC_ENG_FORMAT = "Users' Bookmarks";
static const gchar* DIC_ENG_DESCRIPTION = "This module handles users' bookmarks.";
			    
//------------------------------------------------------------------------------
// macro for "printing" gboolean statement - "TRUE" or "FALSE"
#define PRINT_STATE(state) ( (state) ? "TRUE" : "FALSE" )
//------------------------------------------------------------------------------


//______________________________________________________________________________
// *****************************************************************************
//****************************************** DATA STRUCTURE DEFINITIONS SECTION:
//------------------------------------------------------------------------------
/** \brief Internal data structure of XDXF Engine.
 */
struct _BookData {
        sqlite*                 db;
        
        gchar*                  dict_path;
        EngineStatus            last_error;
        gboolean                auto_free;

        cb_progress             cb_progress_caching;
        gpointer                cb_progress_caching_data;
        gdouble                 cb_progress_caching_seed;

        cb_progress             cb_progress_word_list;
        gpointer                cb_progress_word_list_data;
        gdouble                 cb_progress_word_list_seed;

        cb_progress             cb_progress_word_trans;
        gpointer                cb_progress_word_trans_data;
        gdouble                 cb_progress_word_trans_seed;

        cb_word_list            cb_search_word_list;
        gpointer                cb_search_word_list_data;

        cb_word_translation     cb_search_word_trans;
        gpointer                cb_search_word_trans_data;      
};
typedef struct _BookData        BookData;
//------------------------------------------------------------------------------


//______________________________________________________________________________
// *****************************************************************************
//************************************************ ADDITIONAL FUNCTIONS SECTION:
//------------------------------------------------------------------------------
// returning concrete part of file
//static gchar*           read_file_part(FilePart* part, GnomeVFSHandle* file);
//------------------------------------------------------------------------------
// convert string to proper path name (no filename, no "/" at the ned, file
// exist)
static gchar*           string_to_path(gchar** string);
//------------------------------------------------------------------------------
// tells if file is in XDXF format (file should exist)
//static gboolean         is_xdxf_file(gchar* file); 
//------------------------------------------------------------------------------
// start/stop timers -  returnet -1.0 if we start or seconds passed from start 
// if we want to stop timer
static double           timer(gboolean start, gchar* message);
//------------------------------------------------------------------------------


//______________________________________________________________________________
// *****************************************************************************
//****************************************************** MAIN FUNCTIONS SECTION:
//------------------------------------------------------------------------------
gboolean        bm_engine_add_word(Engine* engine,
                                 gchar*  word,
                                 gchar*  translation);
//------------------------------------------------------------------------------
gboolean        bm_engine_remove_word(Engine* engine,
                                     gchar*  word);  
//------------------------------------------------------------------------------      
gchar*          bm_engine_get_lang_from(Engine* engine);
//------------------------------------------------------------------------------
gchar*          bm_engine_get_lang_to(Engine* engine);
//------------------------------------------------------------------------------
gchar*          bm_engine_get_title(Engine* engine);
//------------------------------------------------------------------------------
gchar*          bm_engine_get_icon_path(Engine* engine);



//------------------------------------------------------------------------------
// implementation of dict_eng_module_check(module,location) function
gboolean        bm_engine_check(gchar* location);
//------------------------------------------------------------------------------
// implementation of dict_eng_module_get_description(module) function
gchar*          bm_engine_description();
//------------------------------------------------------------------------------
// implementation of dict_eng_module_get_format(module) function
gchar*          bm_engine_format();
//------------------------------------------------------------------------------
// implementation of dict_eng_module_get_version(module) function
gchar*          bm_engine_version();
//------------------------------------------------------------------------------
// implementation of dict_eng_module_create(module,location,flags) and
// dict_eng_module_create_ext(module,location,flags) functions
Engine*         bm_engine_create(gchar* location, 
                              EngineOptimizationFlag flags,
                              cb_progress progress_handler,
                              gpointer progress_data,
                              gdouble seed);
//------------------------------------------------------------------------------
// implementation of dict_eng_destroy(engine) function
void            bm_engine_close(Engine* engine);
//------------------------------------------------------------------------------
// implementation of dict_eng_get_location(engine) function
gchar*          bm_engine_location(Engine* engine);
//------------------------------------------------------------------------------
// implementation of dict_eng_optimize(engine) function
void            bm_engine_optimize(Engine* engine);
//------------------------------------------------------------------------------
// implementation of dict_eng_is_optimized( engine ) function
gboolean        bm_engine_is_optimized(Engine* engine);
//------------------------------------------------------------------------------
// implementation of dict_eng_set_auto_free(engine, state) function
void            bm_engine_set_auto_free(Engine* engine, gboolean state);
//------------------------------------------------------------------------------
// implementation of dict_eng_set_callback(engine,signal,c_handler,data) 
// function
gpointer        bm_engine_set_callback(Engine* engine,
                                     gchar* event,
                                     gpointer c_handler,
                                     gpointer user_data);
//------------------------------------------------------------------------------
// implementation of dict_eng_set_progress_seed(engine, signal, val) function
void            bm_engine_set_progress_seed(Engine* engine,
                                         gchar* signal,
                                         gdouble seed);
//------------------------------------------------------------------------------
// implementation ofdict_eng_search_word_list(engine,pattern) function
void            bm_engine_search_word_list(Engine* engine, gchar* pattern);
//------------------------------------------------------------------------------
// implementation of dict_eng_search_word_translation(engine,word) function
void            bm_engine_search_word_translation(Engine* engine, gchar* word);
//------------------------------------------------------------------------------
// implementation of dict_eng_get_last_state(engine) function
EngineStatus    bm_engine_status(Engine* engine);
//------------------------------------------------------------------------------
// implementation of dict_eng_state_message(error) function
gchar*          bm_engine_status_message(EngineStatus error);
//------------------------------------------------------------------------------
// implementation of engine_global_functions(void) function
EngineModule    engine_global_functions();

#ifdef __cplusplus
}
#endif
#endif /* #ifndef _DICTIONARY_ENGINE_STARDICT */
