/******************************************************************************
This file is part of WhiteStork.

WhiteStork is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

WhiteStork is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License 
along with WhiteStork; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Copyright 2006 ComArch S.A.
******************************************************************************/

// header with data structure and function definition for XDXF engine.
// Also Engine API. 
#include <engine_bookmark.h>
#include <string.h>
//------------------------------------------------------------------------------
#define LOGS g_debug
#define g_strlen(string) ( NULL == (string) ? 0 : strlen(string) )
// searching word translation in cache file
static void caching_expat_start(void *data, const char *el, const char **attr);
static void caching_expat_end(void *data, const char *el);
static void caching_expat_text(void *data, const XML_Char *txt, int len);
static gchar* word_translation_cache(XDXFData* data, gchar* word)
{
        gnome_vfs_seek(data->cache, GNOME_VFS_SEEK_START, 0);
        gchar b[DICT_SEARCHING_WORD_TRAN_BUFF_SIZE + 1];
        GnomeVFSFileSize bytes_readed;

        guint word_length = g_strlen(word);
        guint record_length = 0;
        guint trans_offset = 0;
        guint already = 0;
        guint64 readed = 0;
        gchar* buffer = NULL;
        gchar* trans = NULL;
        guint file_size = get_file_size(data->cache);
        if (file_size > 0){
	while(TRUE) {
                gnome_vfs_read(data->cache,
                               b,
                               DICT_SEARCHING_WORD_TRAN_BUFF_SIZE,
                               &bytes_readed
                              );
                guint max_length = (guint)get_max_length(b,bytes_readed);
                readed += max_length;
                buffer = b;
                already = 0;
                while(already < max_length) {
                        memcpy(&record_length, buffer, sizeof(guint));
                        memcpy(&trans_offset,
                                buffer+record_length-2*sizeof(guint),
                                sizeof(guint)
                              );
                        buffer[record_length-sizeof(guint)*2] = '\0';
                        if(((record_length - 3*sizeof(guint)) == word_length) &&
                           (g_utf8_collate(word,buffer+sizeof(guint)) == 0)) {
                                FilePart translation = {0,0};
                                translation.offset = trans_offset;
                                memcpy(&(translation.length),
                                        buffer + record_length - sizeof(guint),
                                        sizeof(guint)
                                      );
                                trans =read_file_part(&translation, data->xdxf);
                                break;
                        };
                        already += record_length;
                        buffer += record_length;
                };

                if( ( bytes_readed < DICT_SEARCHING_WORD_TRAN_BUFF_SIZE ) || 
                      ( readed > (file_size - 3) )) {
                        break;
                      };
                      gnome_vfs_seek(data->cache,
                                     GNOME_VFS_SEEK_CURRENT,
                                     ((gint)max_length) - 
                                          DICT_SEARCHING_WORD_TRAN_BUFF_SIZE
                                    );
        }
	}
        return trans;
}
//------------------------------------------------------------------------------
// searching word translation in xdxf dictionary
static gchar* word_translation_xdxf(XDXFData* data, gchar* word) {
        guint word_length = strlen(word);
        gchar* trans = NULL;
        gnome_vfs_seek(data->xdxf, GNOME_VFS_SEEK_START, 0);
        GnomeVFSResult vfs_result;
        GnomeVFSFileSize bytes_readed = DICT_SEARCHING_WORD_TRAN_BUFF_SIZE;                
        gchar buffer[DICT_SEARCHING_WORD_TRAN_BUFF_SIZE+1];                
        guint64 file_size = get_file_size(data->xdxf);
                //guint word_len = strlen(word); added by me
                
        XML_Parser parser = XML_ParserCreate(NULL);        
        if (!parser) {
                g_warning("%s->%s() Could not open initialize "
                          "XML parser.\n",
                          __FILE__,
                          __FUNCTION__
                         );
                timer(TIMER_STOP,(gchar*)__FUNCTION__);
                return NULL;
        };

        gchar tmp[DICT_MAX_WORD_LENGTH];
        XDXFWordsTransData search_data = {tmp,
                                          word,
                                          word_length,
                                          0,
                                          NULL,
                                          FALSE,
                                          TRUE,
                                          0,
                                          &parser,
                                          FALSE,
                                          data->xdxf
                                         };
        XML_SetElementHandler(parser,
                              search_word_trans_start,
                              search_word_trans_end
                             );
        XML_SetCharacterDataHandler(parser, search_word_trans_text);
                
        XML_SetUserData(parser, &search_data);

        //gdouble last_prog = 0;
        while(TRUE) {
                vfs_result = gnome_vfs_read(data->xdxf,
                                            buffer,
                                            DICT_SEARCHING_WORD_TRAN_BUFF_SIZE,
                                            &bytes_readed
                                           );
                XML_Parse(parser,
                          buffer,
                          bytes_readed,
                          bytes_readed < DICT_SEARCHING_WORD_TRAN_BUFF_SIZE
                         );
                        
                gdouble last_prog = 0.0;
                if(data->cb_progress_word_trans != NULL) {
                        GnomeVFSFileSize act_pos;
                        gnome_vfs_tell(data->xdxf, &act_pos);
                        gdouble progress = 
                                ((gdouble)act_pos)/((gdouble)file_size);
                        if((( (progress - last_prog)/
                              (data->cb_progress_word_trans_seed) ) > 1.0) ||
                            (progress >= 1.0)) {
                                data->
                                cb_progress_word_trans(progress,
                                              data->cb_progress_word_trans_data,
                                              ENGINE_NO_ERROR
                                                      );
                                last_prog = progress;
                        };
                }
                if(bytes_readed < DICT_SEARCHING_WORD_TRAN_BUFF_SIZE) {
                        break;
                }
                if(search_data.cont == FALSE) {
                        break;
                }
        }
        XML_ParserFree(parser);
        trans = search_data.translation; 
        return trans;        
}

//------------------------------------------------------------------------------
// get position of a word in order to add this word to bookmarks
/*GnomeVFSFileSize*/
static FilePart get_adding_position(XDXFData* data, 
			gchar* word)
{
        
	
	g_debug("%s<->", __FUNCTION__);
	gnome_vfs_seek(data->cache, GNOME_VFS_SEEK_START, 0);
        gchar b[DICT_SEARCHING_WORD_TRAN_BUFF_SIZE + 1];
        GnomeVFSFileSize bytes_readed;

        //guint word_length = g_strlen(word);
        guint record_length = 0;
        guint trans_offset = 0;
        guint already = 0;
        guint64 readed = 0;
        gchar* buffer = NULL;
        gchar* trans = NULL;
	
//	g_printf ("\n\ncache: %p\n", data->cache);
	
//	return 0;
	gchar* down_word = g_utf8_strdown(word, -1);
	guint file_size = get_file_size(data->cache);
	if (file_size > 0){
	while(TRUE) {
                gnome_vfs_read(data->cache,
                               b,
                               DICT_SEARCHING_WORD_TRAN_BUFF_SIZE,
                               &bytes_readed
                              );
                guint max_length = (guint)get_max_length(b,bytes_readed);
                readed += max_length;
                buffer = b;
                already = 0;
                while(already < max_length) {
                        memcpy(&record_length, buffer, sizeof(guint));
                        memcpy(&trans_offset,
                                buffer+record_length-2*sizeof(guint),
                                sizeof(guint)
                              );
                        buffer[record_length-sizeof(guint)*2] = '\0';
			gboolean test = 
				g_utf8_collate(buffer+sizeof(guint), down_word);
			g_debug("And here is the buffer content: %s\nCompare result of word %d which is %s", 
				buffer+sizeof(guint), test, down_word);
                        if(/*((record_length - 3*sizeof(guint)) == word_length) &&*/
                           test >= 0)
			{
                                FilePart translation = {0,0};
                                translation.offset = trans_offset;
                                memcpy(&(translation.length),
                                        buffer + record_length - sizeof(guint),
                                        sizeof(guint)
                                      );
                                trans =read_file_part(&translation, data->xdxf);
				return translation/*translation.offset + translation.length*/;  
       
                        };
                        already += record_length;
                        buffer += record_length;
                };

                if( ( bytes_readed < DICT_SEARCHING_WORD_TRAN_BUFF_SIZE ) || 
                      ( readed > (file_size - 3) )) {
                        break;
                      };
                      gnome_vfs_seek(data->cache,
                                     GNOME_VFS_SEEK_CURRENT,
                                     ((gint)max_length) - 
                                          DICT_SEARCHING_WORD_TRAN_BUFF_SIZE
                                    );
        }
	}
	guint64 tmp_offset = get_file_size(data->xdxf) - g_strlen("</xdxf>");
	//g_debug("TMP_OFFSET: %ld", tmp_offset);
	FilePart translation = {tmp_offset,0};
	return /*trans_offset*/translation;
}

//------------------------------------------------------------------------------
// searching word by concrete engine
void bm_engine_search_word_translation(Engine* engine, gchar* word)
{
        g_debug("%s->%s() called.\n"
                "-->PARAM:engine at adress=%p\n"
                "-->PARAM:word=\'%s\'\n",
                 __FILE__,
                 __FUNCTION__,
                 engine,
                 word);
        g_assert(engine != NULL);
        g_assert(word != NULL);
        // start timer for this function
        timer(TIMER_START, (gchar*)__FUNCTION__);
        XDXFData* data = (XDXFData*)(engine->engine_data);
        // if callback is not set, we do not have to search word
        if(data->cb_search_word_trans == NULL) {
                g_warning("%s->%s() callback for Word Translation not set."
                          " Searching aborted.\n",
                          __FILE__,
                          __FUNCTION__
                         );
                timer(TIMER_STOP,(gchar*)__FUNCTION__);
                // do not send any signal, just exit
                return;                
        };
        
        //guint word_length = strlen(word);
        gchar* trans;

        // dictionary is optimized
        if(data->cache != NULL) {
                trans = word_translation_cache(data, word);
        // dictionary is not optimized right now
        } else 
        {        
                trans = word_translation_xdxf(data, word);                
        };

        g_debug("%s->%s() found for word \'%s\' translation:\n\'%s\'\n",
                __FILE__,
                __FUNCTION__,
                word,
                trans
               );
        timer(TIMER_STOP,(gchar*)__FUNCTION__);
        timer(TIMER_START,"callback for returning word's translation START");
        // calling callback for word translation
        data->cb_search_word_trans(trans,
                                   word,
                                   data->cb_search_word_trans_data,
                                   ENGINE_NO_ERROR
                                  );
        timer(TIMER_STOP,"callback for returning word's translation END");
        if(data->auto_free) {
                g_debug("%s->%s() deleting all dynamic data because "
                        "AUTO_FREE=TRUE\n",
                        __FILE__,
                        __FUNCTION__
                       );
                g_free(trans);
        }
        trans = NULL;
	return;
}
//------------------------------------------------------------------------------
gboolean bm_engine_remove_word(Engine* engine,
                             gchar*  word)
{
	g_debug("%s <-> %s()", __FILE__, __FUNCTION__);
	GnomeVFSResult vfs_result;
        XDXFData* data = (XDXFData*)(engine->engine_data);
	GnomeVFSHandle* swap_xdxf;
	vfs_result = 
                gnome_vfs_create(&(swap_xdxf), "/media/mmc1/swap_xdxf.xdxf", 
			GNOME_VFS_OPEN_WRITE | 
			GNOME_VFS_OPEN_READ | GNOME_VFS_OPEN_RANDOM, 
			FALSE, 0666);   
        if(vfs_result != GNOME_VFS_OK)  {
                g_warning("%s->%s() opening dictionary file failed"
                                " due to reason: %s.\n",
                                __FILE__,
                                __FUNCTION__,
                                gnome_vfs_result_to_string(vfs_result)
                                );
		return FALSE;
	}
        
	FilePart position_part = get_adding_position (data, word);
	//GnomeVFSFileSize position = position_part.offset /*+ position_part.length*/;
	//guint64 file_size = get_file_size(data->cache);
	gchar read_buffer[DICT_CACHEING_BUFF_SIZE + 1];
	GnomeVFSFileSize read_size = DICT_CACHEING_BUFF_SIZE;
	GnomeVFSFileSize read_bytes_size = DICT_CACHEING_BUFF_SIZE;
	GnomeVFSFileSize bytes_written;
	gnome_vfs_seek(data->xdxf, GNOME_VFS_SEEK_START, 
 		       		position_part.offset + position_part.length);
	/*Write form position to the end of the file*/
// 	GnomeVFSFileSize position = position_part.offset
	
	while(TRUE)
	{
		g_debug(":Working around in writing the tail:");
 		
		gnome_vfs_read(data->xdxf/*GnomeVFSHandle */,
                		read_buffer/*buffer*/,
                                read_size/*GnomeVFSFileSize*/,
                                &read_bytes_size/*GnomeVFSFileSize*/);
		g_debug("%s", gnome_vfs_result_to_string(vfs_result));
		vfs_result = gnome_vfs_write(swap_xdxf,
                        	read_buffer,
				read_bytes_size,
                                &bytes_written);
		g_debug("%s", gnome_vfs_result_to_string(vfs_result));
		if (read_bytes_size < DICT_CACHEING_BUFF_SIZE) break;
	}
	
	//g_printf (">>> %s %d %s<<<\n", word, position, data->dict_path);
	//gnome_vfs_seek(data->xdxf, GNOME_VFS_SEEK_START, position_part.offset);
	
	gnome_vfs_truncate_handle(data->xdxf, position_part.offset);
	
	read_bytes_size = DICT_CACHEING_BUFF_SIZE;
	read_size = DICT_CACHEING_BUFF_SIZE + 1 ;
	//bytes_written = DICT_CACHEING_BUFF_SIZE;
	gnome_vfs_seek(data->xdxf, GNOME_VFS_SEEK_START, position_part.offset);
	gnome_vfs_seek(swap_xdxf, GNOME_VFS_SEEK_START, 0);
	while(TRUE)
	{
		g_debug(":Working around in writing back the tail:");
		
		gnome_vfs_read(swap_xdxf/*GnomeVFSHandle */,
                		read_buffer/*buffer*/,
                                read_size/*GnomeVFSFileSize*/,
                                &read_bytes_size/*GnomeVFSFileSize*/);
		g_debug("%s", gnome_vfs_result_to_string(vfs_result));
		vfs_result = gnome_vfs_write(data->xdxf,
                        	read_buffer,
				read_bytes_size,
                                &bytes_written);
		g_debug("%s", gnome_vfs_result_to_string(vfs_result));
// 		g_return_val_if_fail(vfs_result != GNOME_VFS_OK, FALSE);
		if (read_bytes_size < DICT_CACHEING_BUFF_SIZE) break;
	}
	gnome_vfs_close(swap_xdxf); 
	gnome_vfs_unlink("/media/mmc1/swap_xdxf.xdxf");
 	gnome_vfs_seek(data->xdxf, GNOME_VFS_SEEK_START, 
  		       	0);
 	gnome_vfs_seek(data->cache, GNOME_VFS_SEEK_START, 
  		       	0);
	gnome_vfs_close(data->cache); 
	gchar* cache_path = g_strconcat(data->dict_path, "/ws_bookmarks.cache", NULL);
	gnome_vfs_unlink(cache_path);
	bm_engine_optimize(engine);
	g_free(cache_path);
	return TRUE;
}

gboolean bm_engine_add_word(Engine* engine,
                         gchar*  word,
                         gchar*  translation) 
{
	g_debug("%s -> %s()", __FILE__, __FUNCTION__);
	
        //guint word_length = strlen(word);
        XDXFData* data = (XDXFData*)(engine->engine_data);
        GnomeVFSResult vfs_result;
	GnomeVFSHandle* swap_xdxf;
	vfs_result = 
                gnome_vfs_create(&(swap_xdxf), "/media/mmc1/swap_xdxf.xdxf", 
			GNOME_VFS_OPEN_WRITE | 
			GNOME_VFS_OPEN_READ | GNOME_VFS_OPEN_RANDOM, 
			FALSE, 0666);   
        if(vfs_result != GNOME_VFS_OK)  {
                g_warning("%s->%s() opening dictionary file failed"
                                " due to reason: %s.\n",
                                __FILE__,
                                __FUNCTION__,
                                gnome_vfs_result_to_string(vfs_result)
                                );
		return FALSE;
	}
	
	
//	guint64 file_size = get_file_size(data->xdxf);
	FilePart position_part = get_adding_position(data, word);
	GnomeVFSFileSize position = 
			position_part.offset/* + position_part.length*/;
	gnome_vfs_seek(data->xdxf, GNOME_VFS_SEEK_START, 
 		       		position);
	gchar read_buffer[DICT_CACHEING_BUFF_SIZE + 1];
	GnomeVFSFileSize read_size = DICT_CACHEING_BUFF_SIZE + 1 ;
	GnomeVFSFileSize read_bytes_size = DICT_CACHEING_BUFF_SIZE;
	GnomeVFSFileSize bytes_written;
	/*Write form position to the end of the file*/
	while(TRUE)
	{
		g_debug(":Working around in writing the tail:");
 		
		vfs_result = gnome_vfs_read(data->xdxf/*GnomeVFSHandle */,
                		read_buffer/*buffer*/,
                                read_size/*GnomeVFSFileSize*/,
                                &read_bytes_size/*GnomeVFSFileSize*/);
		g_debug("%s", gnome_vfs_result_to_string(vfs_result));
		//g_return_val_if_fail(vfs_result != GNOME_VFS_OK, FALSE);
		vfs_result = gnome_vfs_write(swap_xdxf,
                        	read_buffer,
				read_bytes_size,
                                &bytes_written);
		g_debug("%s", gnome_vfs_result_to_string(vfs_result));
		//g_return_val_if_fail(vfs_result != GNOME_VFS_OK, FALSE);
		if (read_bytes_size < DICT_CACHEING_BUFF_SIZE) break;
	}
	/*End write*/
	/*Write new entry*/
	/*gnome_vfs_seek(data->xdxf, GNOME_VFS_SEEK_START, 
		       	position);*/
	gchar* buffer = NULL;
	/*Testing if the adding word is in bookmark file*/
	gchar* search_pattern = g_strconcat("<k>", word, "</k>", NULL);
	//gint search_pattern_len = g_strlen(search_pattern);
	gchar read_word[position_part.length + 1];
	gnome_vfs_seek(data->xdxf, GNOME_VFS_SEEK_START, 
 		       		position_part.offset);
	gnome_vfs_read(data->xdxf/*GnomeVFSHandle */,
                	read_word/*buffer*/,
                        position_part.length/*GnomeVFSFileSize*/,
                        &read_bytes_size/*GnomeVFSFileSize*/);
	gchar* cmp_result = g_strrstr(read_word /*haystack*/,
                                        search_pattern/*needle*/);
	if (cmp_result != NULL)
	{
		
		gchar** tmp = g_strsplit(translation,
                                        "</k>",
                                        2);
		gchar* tmp2 = g_strndup(tmp[1], 
				g_strlen(tmp[1]) - g_strlen("</ar>"));
		gchar* tmp3 = g_strndup(read_word, 
				g_strlen(read_word) - g_strlen("</ar>"));
		buffer = g_strconcat(tmp3, tmp2, "</ar>", NULL);
		g_strfreev(tmp);
		g_free(tmp2);
		g_free(tmp3);
		gnome_vfs_seek(swap_xdxf, GNOME_VFS_SEEK_START, 
 		       	position_part.length);
	}else
	{
		buffer = g_strdup(translation); 
		gnome_vfs_seek(swap_xdxf, GNOME_VFS_SEEK_START, 0);
	}
	/*End of testing*/
	gnome_vfs_truncate_handle(data->xdxf, position_part.offset);
       	gnome_vfs_seek(data->xdxf, GNOME_VFS_SEEK_END, 0);
	GnomeVFSFileSize buffer_size = g_strlen(buffer);
			
	gnome_vfs_write(data->xdxf,
                       	buffer,
			buffer_size,
                        &bytes_written
                        );
	
	read_bytes_size = DICT_CACHEING_BUFF_SIZE;
	read_size = DICT_CACHEING_BUFF_SIZE + 1 ;
	//bytes_written = DICT_CACHEING_BUFF_SIZE;
	//gnome_vfs_seek(data->xdxf, GNOME_VFS_SEEK_START, 0);
	while(TRUE)
	{
		
		
		vfs_result = gnome_vfs_read(swap_xdxf/*GnomeVFSHandle */,
                		read_buffer/*buffer*/,
                                read_size/*GnomeVFSFileSize*/,
                                &read_bytes_size/*GnomeVFSFileSize*/);
		g_debug("%s", gnome_vfs_result_to_string(vfs_result));
		vfs_result = gnome_vfs_write(data->xdxf,
                        	read_buffer,
				read_bytes_size,
                                &bytes_written);
		//g_return_val_if_fail(vfs_result != GNOME_VFS_OK, FALSE);
		g_debug("%s", gnome_vfs_result_to_string(vfs_result));
		//g_debug("%d", bytes_written);
		if (read_bytes_size < DICT_CACHEING_BUFF_SIZE) break;
	}	
	/*End write back*/
	
	gnome_vfs_close(swap_xdxf); 
	gnome_vfs_unlink("/media/mmc1/swap_xdxf.xdxf");
// 	gnome_vfs_close(data->xdxf);
// 	gnome_vfs_open(data->xdxf, data->dict_path, GNOME_VFS_OPEN_READ);
	gnome_vfs_seek(data->xdxf, GNOME_VFS_SEEK_START, 
 		       	0);
	gnome_vfs_seek(data->cache, GNOME_VFS_SEEK_START, 
 		       	0);
	gnome_vfs_close(data->cache); 
	gchar* cache_path = g_strconcat(data->dict_path, "/ws_bookmarks.cache", NULL);
	//gnome_vfs_unlink(cache_path);
	bm_engine_optimize(engine);
	g_free(cache_path);
	return TRUE;
}

static void search_word_trans_start(void *data,
                                    const char *el,
                                    const char **attr
                                   )
{
        XDXFWordsTransData* loc_data = (XDXFWordsTransData*)data;
        if((loc_data->translation != NULL) || !(loc_data->cont)) {
                return;
        };

        if(g_utf8_collate(el,"k") == 0) {
                loc_data->one_word = 1;
        } else if(g_utf8_collate(el,"ar") == 0) {
                loc_data->last_start = 
                                XML_GetCurrentByteIndex(*(loc_data->parser));
        }
}
//------------------------------------------------------------------------------
static void search_word_trans_end(void *data, const char *el)
{
        XDXFWordsTransData* loc_data = (XDXFWordsTransData*)data;
        if((loc_data->translation != NULL) || !(loc_data->cont)) {
                return;
        };

        if(g_utf8_collate(el,"k") == 0) {
                loc_data->one_word = 0;

                gint com = g_utf8_collate(loc_data->last_word, loc_data->word);
                if(com > 0) {
                        loc_data->cont = FALSE;
                        return;
                } else if((loc_data->last_word_length == loc_data->word_len) &&
                          ( com == 0 )
                         )  {
                        loc_data->found = TRUE;
                };       
                // "clearing" buffer for next word        
                loc_data->last_word_length = 0;
        } 
        else if((g_utf8_collate(el,"ar") == 0) && (loc_data->found)) {
                loc_data->found = FALSE;
                loc_data->cont = FALSE;
                gulong last_stop = 
                       (gulong)XML_GetCurrentByteIndex(*(loc_data->parser));
                last_stop += strlen("</ar>");
                FilePart fp = {loc_data->last_start,
                               (last_stop - (loc_data->last_start))
                              };
                loc_data->translation = read_file_part(&fp, loc_data->xdxf);
        }
}
//------------------------------------------------------------------------------
static void search_word_trans_text(void *data, const XML_Char *txt, int len)
{
        XDXFWordsTransData* loc_data = (XDXFWordsTransData*)data;
        if((loc_data->translation != NULL) || !(loc_data->cont))  {
                return;
        };

        if(loc_data->one_word == 1) {
                memcpy(&(loc_data->last_word[loc_data->last_word_length]),
                       (gchar*)txt,
                       len
                      );
                loc_data->last_word_length += (guint)len;
                loc_data->last_word[loc_data->last_word_length] = '\0';
        };
}
//------------------------------------------------------------------------------
void bm_engine_search_word_translation_extended(Engine* engine, gchar* word)
{
        g_debug("%s->%s() called.\n",__FILE__,__FUNCTION__);
}
//------------------------------------------------------------------------------
void bm_engine_set_progress_seed(Engine* engine, gchar* signal, gdouble seed) {
        g_debug("%s->%s() called.\n",__FILE__,__FUNCTION__);
        XDXFData* data = (XDXFData*)(engine->engine_data);
        if(g_ascii_strcasecmp(signal,ENGINE_PROGRESS_OPTIMIZING_SIGNAL) == 0)  {
                data->cb_progress_caching_seed = seed;
                g_debug("%s->%s() sets new seed=%0.2f for for signal "
                        "\"%s\".\n",
                        __FILE__,
                        __FUNCTION__,
                        seed,
                        signal
                       );        
        } 
        else {
                g_warning("%s->%s() unsupported signal"
                          "for progress: %s.\n",
                          __FILE__,
                          __FUNCTION__,
                          signal
                         );
        };
}
//------------------------------------------------------------------------------
gpointer bm_engine_set_callbacks(Engine* engine,
                             gchar* signal,
                             gpointer c_handler,
                             gpointer user_data)
{
	g_debug("%s->%s() called.\n",__FILE__,__FUNCTION__);
        g_assert(engine != NULL);
        g_assert(signal != NULL);
        g_assert(c_handler != NULL);
        XDXFData* data = (XDXFData*)(engine->engine_data);
        if(g_ascii_strcasecmp(signal,ENGINE_PROGRESS_OPTIMIZING_SIGNAL) == 0)  {
                gpointer result = data->cb_progress_caching;
                data->cb_progress_caching = c_handler;
                data->cb_progress_caching_data = user_data;
                g_debug("%s->%s() sets handler for signal \"%s\".\n",
                        __FILE__,
                        __FUNCTION__,
                        signal
                       );
                g_debug("%s->%s() Function at adress =  %d.\n",
                        __FILE__,
                        __FUNCTION__,
                        (guint)c_handler
                       );
                g_debug("%s->%s()     Data at adress =  %d.\n",
                        __FILE__,
                        __FUNCTION__,
                        (guint)user_data
                       );
                return result;                
        }
        else if(g_ascii_strcasecmp(signal, ENGINE_WORD_LIST_SIGNAL) == 0) {
                gpointer result = data->cb_search_word_list;
                data->cb_search_word_list = c_handler;
                data->cb_search_word_list_data = user_data;
                g_debug("%s->%s() sets handler for signal \"%s\".\n",
                        __FILE__,
                        __FUNCTION__,
                        signal
                       );
                g_debug("%s->%s() Function at adress =  %d.\n",
                        __FILE__,
                        __FUNCTION__,
                        (guint)c_handler
                       );
                g_debug("%s->%s()     Data at adress =  %d.\n",
                        __FILE__,
                        __FUNCTION__,
                        (guint)user_data
                       );
                return result;                        
        }
        else if(g_ascii_strcasecmp(signal,
                                   ENGINE_WORD_TRANSLATION_SIGNAL) == 0)  {
                gpointer result = data->cb_search_word_trans;
                data->cb_search_word_trans = c_handler;
                data->cb_search_word_trans_data = user_data;
                g_debug("%s->%s() sets handler for signal \"%s\".\n",
                        __FILE__,
                        __FUNCTION__,
                        signal
                       );
                g_debug("%s->%s() Function at adress =  %d.\n",
                        __FILE__,
                        __FUNCTION__,
                        (guint)c_handler
                       );
                g_debug("%s->%s()     Data at adress =  %d.\n",
                        __FILE__,
                        __FUNCTION__,
                        (guint)user_data
                       );
                return result;                        
        }
        else {
                g_warning("%s->%s() unsupported signal: %s.\n",
                          __FILE__,
                          __FUNCTION__,
                          signal
                         );
                return NULL;
        }
}
//------------------------------------------------------------------------------
void bm_engine_close(Engine* engine)
{
        g_debug("%s->%s() called.\n-->PARAM: engine adress=%p\n",
                __FILE__,
                __FUNCTION__,
                engine);
        if(engine == NULL) {
                g_warning("%s->%s() Trying delete not existed engine.\n",
                           __FILE__,
                           __FUNCTION__
                          );
                return;        
        }
        XDXFData* data = (XDXFData*)(engine->engine_data);
        if(data->cache != NULL) {
                gnome_vfs_close(data->cache);
        };
        if(data->xdxf != NULL) {
                gnome_vfs_close(data->xdxf);
        };
        
        g_free(data->dict_path);
        g_free(data);
        g_free(engine);
        g_debug("%s->%s() engine at adress=%p is deleted.\n",
                __FILE__,
                __FUNCTION__,
                engine);
}
//------------------------------------------------------------------------------
gchar* bm_engine_error_message(EngineStatus error) 
{
        g_debug("%s->%s() called.\n",__FILE__,__FUNCTION__);
        return "Error - not yet implemented.";
}
//------------------------------------------------------------------------------
Engine* bm_engine_create(gchar* location,
                      EngineOptimizationFlag auto_cache,
                      cb_progress progress_handler,
                      gpointer progress_data,
                      gdouble seed)
{
        g_debug("%s->%s() called.\n"
                "-->PARAM:location=\'%s\'\n"
                "-->PARAM:auto_cache=%d\n",
                __FILE__,
                __FUNCTION__,
                location,
                (guint)auto_cache
               );
        timer(TIMER_START,(gchar*)__FUNCTION__);        
        GnomeVFSResult    open_result;

        if(!gnome_vfs_initialized ()) {
                gnome_vfs_init ();                       
        };

        gchar* tmp = g_strdup(location);
        string_to_path(&tmp);

        Engine* result = (Engine*)g_try_malloc(sizeof(Engine));
        result->engine_location = bm_engine_location;
        result->engine_is_optimized = bm_engine_is_optimized;
        result->engine_optimize = bm_engine_optimize;
        result->engine_search_word_list = bm_engine_search_word_list;
        result->engine_search_word_translation = 
                        bm_engine_search_word_translation;

        result->engine_close = bm_engine_close;
        result->engine_status = bm_engine_error;
        result->engine_error_message = bm_engine_error_message;
        result->engine_set_callback = bm_engine_set_callbacks;
        result->engine_set_progress_seed = bm_engine_set_progress_seed;
        result->engine_set_auto_free = bm_engine_set_auto_free;
	// API 0.2
	result->engine_add_word = bm_engine_add_word;
        result->engine_remove_word = bm_engine_remove_word;
	
        XDXFData* data = (XDXFData*)g_try_malloc(sizeof(XDXFData));
        result->engine_data = (gpointer)data;
	
        
        g_debug("%s->%s() opening file...\'%s\'.\n",
                __FILE__,
                __FUNCTION__,
                location
                );
        gchar* tmp2 = g_strconcat(tmp,"/ws_bookmarks.xdxf",NULL);
        open_result = 
                gnome_vfs_open (&(data->xdxf), tmp2, GNOME_VFS_OPEN_READ | 
			GNOME_VFS_OPEN_WRITE | GNOME_VFS_OPEN_RANDOM);
        g_free(tmp2); tmp2 = NULL;
        
        if(open_result != GNOME_VFS_OK)  {
                g_warning("%s->%s() opening dictionary file failed"
                                " due to reason: %s.\n",
                                __FILE__,
                                __FUNCTION__,
                                gnome_vfs_result_to_string(open_result)
                                );
                result->engine_data = NULL;
                g_free(data);
                g_free(result);
                result = NULL;
        }
        else {
                g_debug("%s->%s() opening dictionary file successed.\n",
                        __FILE__,
                        __FUNCTION__
                       );
                data->dict_path = g_strdup(tmp);
                data->cache = NULL;
                data->cb_progress_caching = progress_handler;
                data->cb_progress_caching_data = progress_data;        
                data->cb_progress_caching_seed = seed;        
                data->cb_progress_word_list = NULL;
                data->cb_progress_word_list_data = NULL;
                data->cb_progress_word_list_seed = 0.01;
                data->cb_progress_word_trans = NULL;
                data->cb_progress_word_trans_data = NULL;
                data->cb_progress_word_trans_seed = 0.01;

                data->cb_search_word_list = NULL;
                data->cb_search_word_list_data = NULL;

                data->cb_search_word_trans = NULL;
                data->cb_search_word_trans_data = NULL;

                data->auto_free = FALSE;
                if(auto_cache != ENGINE_NO) {
                        if(auto_cache == ENGINE_REFRESH) {
                                bm_engine_optimize(result);
                        }
                        else if(auto_cache == ENGINE_CREATE) {
			        gchar* cache_path = g_strconcat(data->dict_path,
                                                                "/ws_bookmarks.cache",
                                                                NULL);
				open_result = 
                                        gnome_vfs_open (&(data->cache),
                                                        cache_path,
                                                        GNOME_VFS_OPEN_READ
                                                       );
                                if(open_result != GNOME_VFS_OK) {
                                        bm_engine_optimize(result);
                                };
                                g_free(cache_path); cache_path = NULL;
                        }
                };
        }
        g_free(tmp); tmp = NULL;
        
        timer(TIMER_STOP,(gchar*)__FUNCTION__);
        g_debug("%s->%s() returned Engine at adress=%p\n TO NAPEWNO TEN PLIK",
                __FILE__,
                __FUNCTION__,
                result
               );
        return result;
}
//------------------------------------------------------------------------------
EngineModule engine_global_functions()
{
        g_debug("%s->%s() called.\n",__FILE__,__FUNCTION__);
        EngineModule* result = g_try_new(EngineModule, 1);        
                result->engine_check             = bm_engine_check;
                result->engine_description       = bm_engine_description;
                result->engine_format            = bm_engine_format;
                result->engine_version           = bm_engine_version;
                result->engine_create            = bm_engine_create;
        g_debug("%s->%s() returned EngineModule at adress=%p.\n",
                __FILE__,
                __FUNCTION__,
                result
               );
        return *result;
}
//------------------------------------------------------------------------------
static double timer(gboolean start, gchar* message)
{
        static GArray* stack = NULL;
        static gboolean first_run = TRUE;
        static struct timeval actual_time;
        static struct timeval last_time;
        static struct timeval result;
        static double seconds = 0.0;
        if(first_run) {
                first_run = FALSE;
                stack = g_array_new(TRUE, TRUE, sizeof(struct timeval));
        };        

        if (start) {
                g_debug("XDXF->%s() start counting time for function '%s()'.\n",
                        __FUNCTION__,
                        message
                       );
                g_array_prepend_val(stack, actual_time);
                gettimeofday(&g_array_index(stack, struct timeval, 0),NULL);
                return -1.0;
        }
        // we just want to end some timer - print some information about 
        // working time;
        else {          
                gettimeofday(&actual_time,NULL);
                last_time = g_array_index(stack, struct timeval, 0);
                g_array_remove_index(stack, 0);

                if (actual_time.tv_usec < last_time.tv_usec) {
                        int nsec = (last_time.tv_usec - actual_time.tv_usec) / 
                                                                  (1000000 + 1);
                        last_time.tv_usec -= 1000000 * nsec;
                        last_time.tv_sec += nsec;
                }
                if (actual_time.tv_usec - last_time.tv_usec > 1000000) {
                        int nsec = (last_time.tv_usec - actual_time.tv_usec) / 
                                                                        1000000;
                        last_time.tv_usec += 1000000 * nsec;
                        last_time.tv_sec -= nsec;
                }
                result.tv_sec = actual_time.tv_sec - last_time.tv_sec;
                result.tv_usec = actual_time.tv_usec - last_time.tv_usec;
                seconds = (((double)(result.tv_usec)) / 1e6) +
                                                      ((double)(result.tv_sec));

                g_debug("XDXF->%s() function \'%s()\' was working for: %g [s] "
                        "or %ld [us].\n",
                        __FUNCTION__,
                        message,
                        seconds,
                        ((long)(result.tv_sec*1e6)+(result.tv_usec))
                       );
                // stack is empty so we delete everything
                if(stack->len == 0)   
                {
                        g_array_free(stack, TRUE);
                        first_run = TRUE;
                }
        }
        return seconds;
}
//------------------------------------------------------------------------------
static gchar* read_file_part(FilePart* part, GnomeVFSHandle* file) 
{
        g_debug("%s->%s() called.\n",__FILE__,__FUNCTION__);
        timer(TIMER_START,(gchar*)__FUNCTION__);
        gchar* result = NULL;
        GnomeVFSResult    f_result;
        GnomeVFSFileSize  bytes_read;
        
        f_result = gnome_vfs_seek(file, GNOME_VFS_SEEK_START, part->offset);        
        if(f_result != GNOME_VFS_OK) {
                g_warning("%s->%s() failed. Not possible to seek "
                          "through file!\n",
                          __FILE__,
                          __FUNCTION__
                         );
                timer(TIMER_STOP,(gchar*)__FUNCTION__);
                return result;
                
        };
        result = g_try_malloc((part->length + 1) * sizeof(gchar));
        if(result == NULL) {
                g_warning("%s->%s() failed. Not possible to allocate "
                          "so big memmory chunk!\n",
                          __FILE__,
                          __FUNCTION__
                         );
                timer(TIMER_STOP,(gchar*)__FUNCTION__);
                return result;
        };
        f_result = gnome_vfs_read (file, result, part->length, &bytes_read);
        if((f_result != GNOME_VFS_OK) ||
                (((gulong)bytes_read) != part->length)) {
                g_debug("%s->%s() failed. Not possible to read from "
                        "file!\n",
                        __FILE__,
                        __FUNCTION__
                       );
                timer(TIMER_STOP,(gchar*)__FUNCTION__);
                g_free(result); result = NULL;
                return result;                
        };
        result[part->length] = '\0';

        g_debug("%s->%s() returned string=\n\'%s\'.\n",
                __FILE__,
                __FUNCTION__,
                result
               );
        timer(TIMER_STOP,(gchar*)__FUNCTION__);
        return result;
}
//------------------------------------------------------------------------------
/** Translate given string to proper full file path. Function works "in-situ". 
* It means that string is translated and replaced by proper full file path if 
* this file path exists, or this string is cleared and setted to NULL, if string
* was representing wrong path. This function is static - to use only 
* within this module.
*
* @param string :: <b><i>gchar**</i></b> - pointer to pointer to string 
*       representing file path - it will be replaced by a proper filepath.
*       It should be path to directory or file, ended with "/" or not, absolute 
*       or not
* @return <b><i>gchar*</i></b> - pointer to new full file path, it is the same
*       as string[0] in fact and it is returned only for abillity of nesting of 
* functions by pointer to string identyfying full file path
*/
static gchar* string_to_path(gchar** string) {
        g_debug("%s->%s() called.\n\
                 -->PARAM:string=\'%s\'\n",
                 __FILE__,
                 __FUNCTION__,
                 string[0]
               );
        gchar* arg = string[0];
        gchar* new = NULL;
        // cleaning from leading and trailing whitespaces
        g_strstrip(arg);        
         // add current directory if this is not absolute directory
        if (!g_path_is_absolute(arg)) {
                gchar* tmp = g_get_current_dir();
                new = g_strconcat(tmp,"/",arg,NULL);
                g_free(arg); arg = new; new = NULL;
        };
        // this is not a directory
        if (!g_file_test(arg, G_FILE_TEST_IS_DIR)) {        
                // if this is wrong filepath, string was wrong
                if (!g_file_test(arg, G_FILE_TEST_IS_REGULAR)) {        
                        g_free(arg);
                        new = NULL;
                }
                //if this is a file, remove filename
                else
                {   
                        new = g_path_get_dirname (arg);
                        g_free(arg);
                }
        }
        // this is a directory
        else {   
                // remove suffix "/" if neded...     
                if (g_str_has_suffix(arg,"/") ) {        
                        new = g_path_get_dirname (arg);
                        g_free(arg);
                }
                else {
                        new = arg;
                }
        };
        // now in new should be proper filepath, if not, string was wrong
        if (!g_file_test(new, G_FILE_TEST_IS_DIR))  {        
                // if that directory does not exist, passed string wasn't proper       
                g_free(new);
                new = NULL;
        };
        // replace string under passed address
        string[0] = new;
        g_debug("%s->%s() returned string=\'%s\'\n",
                __FILE__,
                __FUNCTION__,
                string[0]
               );
        return new;
}
//------------------------------------------------------------------------------
static gboolean is_bm_file(gchar* file) {
         g_debug("%s->%s() called.\n\
                 -->PARAM:file=\'%s\'\n",
                 __FILE__,
                 __FUNCTION__,
                 file
               );
        gboolean                result = TRUE;
        GnomeVFSHandle*         fd = NULL;
        GnomeVFSResult          file_result;
        GnomeVFSFileSize        bytes_read;

        if(!gnome_vfs_initialized ()) {
                gnome_vfs_init ();                       
        };

        file_result = gnome_vfs_open (&fd, file, GNOME_VFS_OPEN_READ);
        if(file_result != GNOME_VFS_OK) {
                g_warning("%s->%s() Could not open the file.\n",
                          __FILE__,
                          __FUNCTION__
                         );
                return FALSE;
        };

        XML_Parser p = XML_ParserCreate(NULL);        
        if (!p) {
                g_warning("%s->%s() Could not open initialize "
                          "XML parser.\n",
                          __FILE__,
                          __FUNCTION__
                         );
                gnome_vfs_close(fd);
                return FALSE;
        };
        XML_SetElementHandler(p, is_bm_file_start, is_bm_file_end);
        XDXFCheckingData user_data = {TRUE, FALSE, 0};
        XML_SetUserData(p, &user_data);
        gchar buffer[DICT_CACHEING_BUFF_SIZE];        

        guint loop_count = 0;
        while(TRUE) {
                file_result = gnome_vfs_read (fd,
                                              buffer,
                                              DICT_CACHEING_BUFF_SIZE,
                                              &bytes_read
                                             );
                if  (file_result != GNOME_VFS_OK) {
                        result = FALSE;
                        g_warning("%s->%s() Could not read enought from"
                                  " file.\n",
                                  __FILE__,
                                  __FUNCTION__
                                 );
                        break;
                };               
                if (! XML_Parse(p,
                                buffer,
                                (gulong)bytes_read,
                                ((gulong)bytes_read) < DICT_CACHEING_BUFF_SIZE
                               ) ) {
                        result = FALSE;
                        g_warning("%s->%s() Could not parse file.\n",
                                  __FILE__,
                                  __FUNCTION__
                                 );
                        break;
                };
                if (user_data.further == FALSE) {
                        result = user_data.good;
                        g_debug("%s->%s() statement: location is "
                                "compatible with this module, is %s\n",
                                __FILE__,
                                __FUNCTION__,
                                PRINT_STATE(result)
                               );
                        break;
                };
                if (loop_count > 1) {
                        result = FALSE;
                        g_debug("%s->%s() Wrong file format.\n",
                                __FILE__,
                                __FUNCTION__
                               );
                        break;
                };
                loop_count++;
        }

        gnome_vfs_close(fd);
        XML_ParserFree(p);
        g_debug("%s->%s() returned bool statement=%s.\n",
                __FILE__,
                __FUNCTION__,
                PRINT_STATE(result)
               );
        return result;
}
//------------------------------------------------------------------------------
static void is_bm_file_start(void *data, const char *el, const char **attr) 
{
         XDXFCheckingData* user_data = (XDXFCheckingData*)data;
        if (user_data->deep == 0) {
                if (g_utf8_collate (el,"xdxf") != 0) {
                        user_data->good = FALSE;                
                }
                else {
                        user_data->good = TRUE;
                }
                user_data->further = FALSE;        
        }
        user_data->deep++;
}
//------------------------------------------------------------------------------
static void is_bm_file_end(void *data, const char *el) 
{
        // clear as far as in this callback is nothing to do
}
//------------------------------------------------------------------------------
EngineStatus bm_engine_error(Engine* engine) 
{
        g_debug("%s->%s() called.\n",__FILE__,__FUNCTION__);
        XDXFData* data = (XDXFData*)(engine->engine_data);
        g_debug("%s->%s() returned error code: %d\n",
                __FILE__,
                __FUNCTION__,
                (gint)(data->last_error)
               );        
        return data->last_error;
}
//------------------------------------------------------------------------------
static void caching_expat_start(void *data, const char *el, const char **attr) {
        XDXFCacheData* loc_data = (XDXFCacheData*)data;
        if(g_utf8_collate(el,"ar") == 0)  {
                loc_data->last_start = 
                                     XML_GetCurrentByteIndex(loc_data->parser);
        }
        else if(g_utf8_collate(el,"k") == 0) {
                loc_data->state = 1;
        }
        else {
                loc_data->state = 0;
        }
}
//------------------------------------------------------------------------------
static void caching_expat_end(void *data, const char *el) {
        XDXFCacheData* loc_data = (XDXFCacheData*)data;
        loc_data->last_stop = XML_GetCurrentByteIndex(loc_data->parser);
        
        static guint record_length;
        static guint start;
        static guint length;
        static guint buffer_length;

        if((g_utf8_collate("k",el) == 0) &&
                (loc_data->state == 1))  {
                loc_data->state = 2;
        }
        else if((g_utf8_collate("ar",el) == 0) &&
                        (loc_data->state == 2)) {
                buffer_length = loc_data->buffer_length;
                record_length = sizeof(guint)*3 + loc_data->buffer_length;
                start = loc_data->last_start;
                length = loc_data->last_stop + strlen("</ar>") - 
                         loc_data->last_start;
                
                gboolean error_writting = FALSE;
                GnomeVFSFileSize bytes_written;
                GnomeVFSResult vfs_result;
                vfs_result = gnome_vfs_write(loc_data->cache,
                                             &record_length,
                                             sizeof(guint),
                                             &bytes_written
                                            );
                        if(vfs_result != GNOME_VFS_OK) error_writting = TRUE;
                vfs_result = gnome_vfs_write(loc_data->cache,
                                             loc_data->buffer,
                                             loc_data->buffer_length,
                                             &bytes_written
                                            );
                        if(vfs_result != GNOME_VFS_OK) error_writting = TRUE;
                vfs_result = gnome_vfs_write(loc_data->cache,
                                             &start,
                                             sizeof(guint),
                                             &bytes_written
                                            );
                        if(vfs_result != GNOME_VFS_OK) error_writting = TRUE;
                vfs_result = gnome_vfs_write(loc_data->cache,
                                             &length,
                                             sizeof(guint),
                                             &bytes_written
                                            );
                        if(vfs_result != GNOME_VFS_OK) error_writting = TRUE;
                
                loc_data->buffer[0] = '\0';
                loc_data->buffer_length = 0;
                loc_data->state = 0;
        };
}
//------------------------------------------------------------------------------
static void caching_expat_text(void *data, const XML_Char *txt, int len) {
        XDXFCacheData* loc_data = (XDXFCacheData*)data;

        if(loc_data->state == 1) {
                memcpy(&(loc_data->buffer[loc_data->buffer_length]),
                       (gchar*)txt,
                       len
                      );
                loc_data->buffer_length += (long)len;
                loc_data->buffer[loc_data->buffer_length] = '\0';
        };
}
//------------------------------------------------------------------------------
static guint64 get_file_size(GnomeVFSHandle* file)
{
        guint64 result = 0;
        guint64 old_pos = 0;
        gnome_vfs_tell(file, &old_pos);

        if( gnome_vfs_seek(file, GNOME_VFS_SEEK_END, 0) != GNOME_VFS_OK) {
                return 0;
        }

        if( gnome_vfs_tell(file, &result) != GNOME_VFS_OK) {
                result = 0;
        }
        
        gnome_vfs_seek(file, GNOME_VFS_SEEK_START, old_pos);
        return result;
}
//------------------------------------------------------------------------------
/*
static gboolean is_Bookmark_file(gchar* file) {
        LOGS("Bookmark/%s->%s() called.\n\
                 -->PARAM:file=\'%s\'\n",
                 (gchar*)__FILE__,
                 (gchar*)__FUNCTION__,
                 file
               );
	
	GnomeVFSHandle *handle;
	
	GnomeVFSResult result = gnome_vfs_open (&handle, file, GNOME_VFS_OPEN_READ);
			
        if(result != GNOME_VFS_OK) {
                LOGS("File Access Error.\n");
                return FALSE;
        }

        if( g_strrstr (file, "ws_bookmarks") == NULL) {
                LOGS("Wrong file! Not a bookmarks' database.\n");
                return FALSE;
        }
        return TRUE;
}*/

void bm_engine_optimize(Engine* engine)
{
        g_debug("%s->%s() called for engine at adress=%p\n",
                __FILE__,
                __FUNCTION__,
                engine
               );
        timer(TIMER_START,(gchar*)__FUNCTION__);   
        GnomeVFSResult vfs_result;
        XDXFData* data = (XDXFData*)(engine->engine_data);
	g_debug("data->dict_path %s", data->dict_path);
	gchar* cache_path = g_strconcat(data->dict_path,"/ws_bookmarks.cache",NULL);
	vfs_result = gnome_vfs_create(&(data->cache),
                                      cache_path,
                                      GNOME_VFS_OPEN_WRITE,
                                      FALSE,
                                      0666
                                     );
	if(vfs_result != GNOME_VFS_OK)  {
               data->cache = NULL;
               g_warning("%s->%s().Could not create new cache file: %s.\n",
                          __FILE__,
                          __FUNCTION__,
                          cache_path
                         );
	}
        else {
                XDXFCacheData* c_data = 
                            (XDXFCacheData*)g_try_malloc(sizeof(XDXFCacheData));
                c_data->parser = XML_ParserCreate(NULL);
	        c_data->cache = data->cache;
	        c_data->buffer = 
                    (gchar*)g_try_malloc(sizeof(gchar)*DICT_CACHEING_BUFF_SIZE);
	        c_data->buffer_length = 0;
                c_data->last_start = 0;
                c_data->last_stop = 0;
                c_data->last_length = 0;
                guint64 file_size = get_file_size(data->xdxf);
//                 g_debug("%s->%s(): caching dictionaries size is %.2f kB "
//                         "[%d bytes = %.2f MB].\n",
//                         __FILE__,
//                         __FUNCTION__,
//                         ((gdouble)file_size)/1024.0,
//                         file_size,
//                         ((gdouble)file_size)/(1024.0*1024.0)
//                        );

                XML_SetUserData(c_data->parser, (gpointer)c_data);
	        XML_SetElementHandler(c_data->parser,
                                      caching_expat_start,
                                      caching_expat_end
                                     );
		XML_SetCharacterDataHandler(c_data->parser, caching_expat_text);
                GnomeVFSFileSize bytes_readed = DICT_CACHEING_BUFF_SIZE;
		gchar b[DICT_CACHEING_BUFF_SIZE + 1];
		gdouble last_prog = 0;
		while(TRUE) {
                        vfs_result = gnome_vfs_read(data->xdxf,
                                                    b,
                                                    DICT_CACHEING_BUFF_SIZE,
                                                    &bytes_readed
                                                   );
		        XML_Parse(c_data->parser,
                                  b,
                                  bytes_readed,
                                  bytes_readed < DICT_CACHEING_BUFF_SIZE
                                 );
                        if(data->cb_progress_caching != NULL) {
                                GnomeVFSFileSize act_pos;
                                gnome_vfs_tell(data->xdxf, &act_pos);
                                gdouble progress = ((gdouble)act_pos)/
                                                           ((gdouble)file_size);
				if((( (progress - last_prog) /
                                    (data->cb_progress_caching_seed) ) > 1.0) ||
                                   (progress >= 1.0)) {
                                        data->cb_progress_caching(
                                                 progress,
                                                 data->cb_progress_caching_data,
                                                 ENGINE_NO_ERROR
                                                                 );
                                        last_prog = progress;
				};
                        }
                        if(bytes_readed < DICT_CACHEING_BUFF_SIZE) break;
                }
		g_free(c_data->buffer);
                g_free(c_data);
        }

        vfs_result = gnome_vfs_close(data->cache);
	vfs_result = gnome_vfs_open(&(data->cache),
                                    cache_path,
                                    GNOME_VFS_OPEN_READ
                                   );
	g_free(cache_path); cache_path = NULL;
	timer(TIMER_STOP,(gchar*)__FUNCTION__);
        g_debug("%s->%s()'s work finished.\n",__FILE__,__FUNCTION__);
}
//------------------------------------------------------------------------------
gboolean bm_engine_check(gchar* location) 
{        
        g_debug("%s->%s() called.\n-->PARAM:location=\'%s\'\n",
                __FILE__,
                __FUNCTION__,
                location
               );
        timer(TIMER_START,(gchar*)__FUNCTION__);
        gboolean result = TRUE;        
        gchar* filepath = g_strdup(location);
        gchar* tmp = NULL;
        
        string_to_path(&filepath);
        if (filepath == NULL) {
                result = FALSE;
                g_warning("%s->%s() location \'%s\' is not a proper "
                          "path!\n",
                          __FILE__,
                          __FUNCTION__,
                          location
                         );
        }
        else {
                tmp = g_strconcat(filepath,"/ws_bookmarks.xdxf",NULL);
                g_free(filepath);
                filepath = tmp;
                tmp = NULL;

                g_debug("%s->%s() finnal file to check is: %s\n",
                        __FILE__,
                        __FUNCTION__,
                        filepath
                       );
                if (!g_file_test(filepath, G_FILE_TEST_IS_REGULAR)) {
                        g_warning("%s->%s() file \'%s\' does not "
                                  "exists!\n",
                                  __FILE__,
                                  __FUNCTION__,
                                  filepath
                                 );
                        result = FALSE;
                };
        };
        if (result != FALSE) {
                result = is_bm_file(filepath);
        };

        g_free(filepath);
        timer(TIMER_STOP,(gchar*)__FUNCTION__);
        g_debug("%s->%s() returned bool statement=%s.\n",
                __FILE__,
                __FUNCTION__,
                PRINT_STATE(result)
               );
        return result;
}


//------------------------------------------------------------------------------
static guint get_max_length(gchar* a, guint length) 
{
        gchar* b = a;
        guint len = 0;
        guint n = 0;
        memcpy(&n,b,sizeof(guint));
        while((len + n) <= (length - 4)) {
                len += n;
                b = b + n;
                memcpy(&n,b,sizeof(guint));
        }
        return len;
}
//------------------------------------------------------------------------------                                                                                     // finished functions:
void bm_engine_set_auto_free(Engine* engine, gboolean state) 
{
        g_debug("%s->%s() called.\n"
                "-->PARAM:engine at adress=%p\n"
                "-->PARAM:state=%s\n",
                __FILE__,
                __FUNCTION__,
                engine,
                PRINT_STATE(state)
               );
        g_assert(engine != NULL);
        XDXFData* data = (XDXFData*)(engine->engine_data);
        data->auto_free = state;
        g_debug("%s->%s() Current auto_free is %s\n",
                __FILE__,
                __FUNCTION__,
                PRINT_STATE(data->auto_free)
               );
}
//------------------------------------------------------------------------------
gchar* bm_engine_version() 
{
        g_debug("%s->%s() called.\n",__FILE__,__FUNCTION__);
        gchar* result = g_strdup(DIC_ENG_VERSION);
        g_debug("%s->%s() return string=%s\n",
                __FILE__,
                __FUNCTION__,
                result
               );
        return result;
}
//------------------------------------------------------------------------------
gchar* bm_engine_format() 
{
	LOGS("Bookmark/%s->%s() called.\n",(gchar*)__FILE__,(gchar*)__FUNCTION__);
	gchar* result = g_strdup(DIC_ENG_FORMAT);
	LOGS("Bookmark/%s->%s() return string=%s\n",
	     (gchar*)__FILE__,
	     (gchar*)__FUNCTION__,
	     result
	    );
	return result;
}
//------------------------------------------------------------------------------
gchar* bm_engine_description() 
{
	LOGS("Bookmark/%s->%s() called.\n",(gchar*)__FILE__,(gchar*)__FUNCTION__);
	gchar* result = g_strdup(DIC_ENG_DESCRIPTION);
	LOGS("Bookmark/%s->%s() return string=%s\n",
	     (gchar*)__FILE__,
	     (gchar*)__FUNCTION__,
	     result
	    );
	return result;
}
//------------------------------------------------------------------------------
gboolean bm_engine_is_optimized(Engine* engine) 
{
        g_debug("%s->%s() called.\n-->PARAM: engine adress=%p\n",
                __FILE__,
                __FUNCTION__,
                engine
               );
        g_assert(engine != NULL);                
        XDXFData* data = (XDXFData*)(engine->engine_data);
        gboolean result = (data->cache != NULL);
        g_debug("%s->%s() returned bool statement=%s.\n",
                __FILE__,
                __FUNCTION__,
                PRINT_STATE(result)
               );
        return result;
}
//------------------------------------------------------------------------------
gchar* bm_engine_location(Engine* engine)
{
        g_debug("%s->%s() called.\n-->PARAM: engine adress=%p\n",
                __FILE__,
                __FUNCTION__,
                engine
               );
        g_assert(engine != NULL);
        XDXFData* data = (XDXFData*)(engine->engine_data);
        gchar* result;
        if(data->auto_free) {
                result = data->dict_path;
        }
        else {
                result = g_strdup(data->dict_path);
        }

        g_debug("%s->%s() returned string=%s\n",
                __FILE__,
                __FUNCTION__,
                result
               );
        return result;
}
//------------------------------------------------------------------------------
static void search_word_list_start(void *data,
                                   const char *el,
                                   const char **attr
                                  )
{
        XDXFWordsListData* loc_data = (XDXFWordsListData*)data;
        if(g_utf8_collate(el,"k") == 0) {
                loc_data->one_word = 1;
        };
}
//------------------------------------------------------------------------------
static void search_word_list_end(void *data, const char *el)
{
        XDXFWordsListData* loc_data = (XDXFWordsListData*)data;
        if(g_utf8_collate(el,"k") == 0) {
                loc_data->one_word = 0;
        }
        else {
                return;
        }
        static  gboolean any_found = FALSE;
                        gboolean matched   = FALSE;

        if(( loc_data->last_word_length >= loc_data->pattern_len ) &&
           (g_ascii_strncasecmp(loc_data->last_word,
                                loc_data->pattern,
                                loc_data->pattern_len) == 0)) {
                matched = TRUE;
                any_found = TRUE;
                gchar* new = g_strdup(loc_data->last_word);
                g_array_append_val((loc_data->result), new);
                g_debug("New Word for pattern \"%s\" found: %s\n",
                        loc_data->pattern,
                        new
                       );
        };
        // "clearing" buffer for next word        
        loc_data->last_word_length = 0;
        // if we passed words matching -> ends
        if(any_found && !matched) {
                loc_data->cont = FALSE;
        };
        matched = FALSE;
        any_found = FALSE;
}
//------------------------------------------------------------------------------
static void search_word_list_text(void *data, const XML_Char *txt, int len) 
{
        XDXFWordsListData* loc_data = (XDXFWordsListData*)data;

        if(loc_data->one_word == 1) {
                memcpy(&(loc_data->last_word[loc_data->last_word_length]),
                       (gchar*)txt,
                       len
                      );
                loc_data->last_word_length += (guint)len;
                loc_data->last_word[loc_data->last_word_length] = '\0';
        };
}
//------------------------------------------------------------------------------
// return translation of word using cache file
static void word_list_cache(XDXFData* data, gchar* pattern, GArray* result) {
        gnome_vfs_seek(data->cache, GNOME_VFS_SEEK_START, 0);        
//        GnomeVFSResult vfs_result;
	GnomeVFSFileSize bytes_readed;
        guint64 file_size = get_file_size(data->cache);
	if (file_size > 0){
	guint pattern_len;
	gchar buffer[DICT_SEARCHING_WORD_LIST_BUFF_SIZE];
        gchar* buf;
	guint record_length = 0;
        guint already = 0;
        guint max_length = 0;
	pattern_len = g_strlen(pattern);
	g_strstrip(pattern);
	if ((int)(pattern[0]) == 42 && pattern[1] == '\0') //asterix?
	{
		gchar pattern_copy[2] = "A\0";
	
       		pattern_len = g_strlen(pattern_copy);
       		while (TRUE)
		{
		
        		record_length = 0;
        		already = 0;
			max_length = 0;
	//gnome_vfs_seek(data->cache, GNOME_VFS_SEEK_START, 0);        
	//
			while(TRUE)  {
                		gnome_vfs_read(data->cache,
                               		buffer,
                               		DICT_SEARCHING_WORD_LIST_BUFF_SIZE,
                               		&bytes_readed
                              		);
                        
                	max_length = get_max_length(buffer, (guint)bytes_readed);
                	already += max_length;
                	buf = buffer;

                	guint how_far = 0;
                	while(how_far < max_length) {
                        	memcpy(&record_length, buf, sizeof(guint));
                        	if(
				((record_length - 3*sizeof(guint)) >= pattern_len) &&
                           	(g_ascii_strncasecmp(buf + sizeof(guint),
                                                 pattern_copy,
                                                 pattern_len
                                                ) == 0 ) ) {
                                	gchar* new = 
						g_strndup(buf + sizeof(guint),
                                                       record_length - 
                                                               3*sizeof(guint));
                                	g_array_append_val(result, new);
                                	g_debug(
					"New Word for pattern \"%s\" found: "
                                        "%s\n",
                                        pattern_copy,
                                        new
                                       );
                        	};
                        	how_far += record_length;
                        	buf = buf + record_length;
                        }
                	if((bytes_readed < DICT_SEARCHING_WORD_LIST_BUFF_SIZE)|| 
                     		(already > (file_size -3))) {
                        	break;
                     	}
				gnome_vfs_seek(data->cache, 
					GNOME_VFS_SEEK_CURRENT,
                                    	((gint)max_length) - 
                                             DICT_SEARCHING_WORD_LIST_BUFF_SIZE
                                   );
        		}
			pattern_copy[0] = (gchar)((gint) pattern_copy[0] + 1);
			
			g_debug("now this is pattern %d ", (gint) pattern_copy[0]);
			if ((gint) pattern_copy[0] > 90) break;
			gnome_vfs_seek(data->cache, GNOME_VFS_SEEK_START, 0);
		}
	}else
	{
		while(TRUE)  {
                	gnome_vfs_read(data->cache,
                               		buffer,
                               		DICT_SEARCHING_WORD_LIST_BUFF_SIZE,
                               		&bytes_readed
                              		);
                        
                	max_length = get_max_length(buffer, (guint)bytes_readed);
                	already += max_length;
                	buf = buffer;

                	guint how_far = 0;
                	while(how_far < max_length) {
                        	memcpy(&record_length, buf, sizeof(guint));
                        	if(
				((record_length - 3*sizeof(guint)) >= pattern_len) &&
                           	(g_ascii_strncasecmp(buf + sizeof(guint),
                                                 pattern,
                                                 pattern_len
                                                ) == 0 ) ) {
                                	gchar* new = 
						g_strndup(buf + sizeof(guint),
                                                       record_length - 
                                                               3*sizeof(guint));
                                	g_array_append_val(result, new);
                                	g_debug(
					"New Word for pattern \"%s\" found: "
                                        "%s\n",
                                        pattern,
                                        new
                                       );
                        	};
                        	how_far += record_length;
                        	buf = buf + record_length;
                        }
                	if((bytes_readed < DICT_SEARCHING_WORD_LIST_BUFF_SIZE)|| 
                     		(already > (file_size -3)) ) {
         	
				break;
                     	}
				gnome_vfs_seek(data->cache, 
					GNOME_VFS_SEEK_CURRENT,
                                    	((gint)max_length) - 
                                             DICT_SEARCHING_WORD_LIST_BUFF_SIZE
                                   );
        	}	
	}
	}
	timer(TIMER_STOP,(gchar*)__FUNCTION__);
        timer(TIMER_START,"callback for returning words list START");                
        data->cb_search_word_list(result,
                                  pattern,
                                  data->cb_search_word_list_data,
                                  ENGINE_NO_ERROR
                                 );
        timer(TIMER_STOP,"callback for returning words list END");        
}
//------------------------------------------------------------------------------
// return translation of word but using only xdxf dictionary file
static void word_list_xdxf(XDXFData* data, gchar* pattern, GArray* result) {
        gnome_vfs_seek(data->xdxf, GNOME_VFS_SEEK_START, 0);
        GnomeVFSResult vfs_result;
        GnomeVFSFileSize bytes_readed = DICT_SEARCHING_WORD_LIST_BUFF_SIZE;                
        gchar buffer[DICT_SEARCHING_WORD_LIST_BUFF_SIZE+1];                
        guint64 file_size = get_file_size(data->xdxf);
	guint pattern_len;
               
        XML_Parser parser = XML_ParserCreate(NULL);        
        if (!parser) {
                g_warning("%s->%s() Could not open initialize XML "
                          "parser.\n",
                          __FILE__,
                          __FUNCTION__
                         );
                timer(TIMER_STOP,(gchar*)__FUNCTION__);
                return;
        };
	
	
	gchar tmp[DICT_MAX_WORD_LENGTH];
        XML_SetElementHandler(parser,
                              search_word_list_start,
                              search_word_list_end
                             );
        XML_SetCharacterDataHandler(parser, search_word_list_text);
        
        // buffer for single word 
        // pattern to search 
        // length of pattern 
        // actal length of readed word 
        // array to append words 
        // continuation of the same word 
        // continue of searching?
        XDXFWordsListData search_data = {tmp,
                                         pattern,
                                         pattern_len,
                                         0,
                                         result,
                                         0,
                                         TRUE
                                        }; 
        XML_SetUserData(parser, &search_data);

        gdouble last_prog = 0;
       
	while(TRUE) {
                vfs_result = gnome_vfs_read(data->xdxf,
                                            buffer,
                                            DICT_SEARCHING_WORD_LIST_BUFF_SIZE,
                                            &bytes_readed
                                           );
                XML_Parse(parser,
                          buffer,
                          bytes_readed,
                          bytes_readed < DICT_SEARCHING_WORD_LIST_BUFF_SIZE
                         );
                        
                if(data->cb_progress_word_list != NULL) {
                        GnomeVFSFileSize act_pos;
                        gnome_vfs_tell(data->xdxf, &act_pos);
                        gdouble progress = ((gdouble)act_pos)/
                                                           ((gdouble)file_size);
                        if((((progress - last_prog)/
                            (data->cb_progress_word_list_seed)) > 1.0) ||
                            (progress >= 1.0)) {
                                data->cb_progress_word_list(
                                               progress,
                                               data->cb_progress_word_list_data,
                                               ENGINE_NO_ERROR
                                                           );
                                last_prog = progress;
                        };
                }
                if(bytes_readed < DICT_SEARCHING_WORD_LIST_BUFF_SIZE) {
                        break;
                }
                if((search_data.cont) == FALSE) {
                        g_debug("%s->%s() We found every words matching "
                                "pattern \"%s\". Abort further searching.\n",
                                __FILE__,
                                __FUNCTION__,
                                pattern
                               );
                        break;
                }
        }
	 
	
     	
	
	XML_ParserFree(parser);
	
        timer(TIMER_STOP,(gchar*)__FUNCTION__);
        timer(TIMER_START,"callback for returning words list START");
        data->cb_search_word_list(result,
                                  pattern,
                                  data->cb_search_word_list_data,
                                  ENGINE_NO_ERROR
                                 );
        timer(TIMER_STOP,"callback for returning words list END");        
}
//------------------------------------------------------------------------------
void bm_engine_search_word_list(Engine* engine, gchar* pattern)
{
        g_debug("%s->%s() called. Searching words list\n"
                "-->PARAM:engine at adress=%p\n"
                "-->PARAM:pattern=\"%s\"\n",
                __FILE__,
                __FUNCTION__,
                engine,
                pattern
               );
        g_assert(engine != NULL);
        g_assert(pattern != NULL);

        timer(TIMER_START,(gchar*)__FUNCTION__);
        XDXFData* data = (XDXFData*)(engine->engine_data);
	if(data->cb_search_word_list == NULL) {
                g_warning("%s->%s() callback for Word List not set. "
                          "Searching aborted.\n",
                          __FILE__,
                          __FUNCTION__
                         );
                timer(TIMER_STOP,(gchar*)__FUNCTION__);
                return;
        };
	
	
        GArray* result = g_array_new(TRUE,FALSE,sizeof(gchar*));
        // dictionary is optimized so search in cache file
        if(data->cache != NULL) {        
                word_list_cache(data, pattern, result);
        }
        // dictionary is not optimized so search directly fom XDXF file
        else {          
                word_list_xdxf(data, pattern, result);
        };
        
        if(data->auto_free == TRUE)  {
                g_debug("%s->%s() deleting all dynamic data because "
                        "AUTO_FREE=TRUE\n",
                        __FILE__,
                        __FUNCTION__
                       );
                gchar* tmp;
                guint i = 0;
                while((tmp = g_array_index(result, gchar*, i)) != NULL) 
                {
                        g_free(tmp); tmp = NULL;
                        i++;
                }
                g_array_free(result, TRUE);
        };
        g_debug("%s->%s() finished definately its work.\n",
                __FILE__,
                __FUNCTION__
               );
	return;
}
