/*******************************************************************************
This file is part of WhiteStork.

WhiteStork is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

WhiteStork is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License 
along with WhiteStork; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Copyright 2006 ComArch S.A.
*******************************************************************************/
#ifndef _WS_MANAGER
#define _WS_MANAGER

#include <dictionary_engine.h>
#include <glib-object.h>
#include <gmodule.h>
#include <ws_dbus.h>

#ifdef __cplusplus
extern "C" {
#endif

#if SQLITE==0
	#define LIBRARY "/usr/lib/libsqlite.so.0"
#elif SQLITE==3
	#define LIBRARY "/usr/lib/libsqlite3.so.0"
#endif

#define g_strlen(string) ( NULL == (string) ? 0 : strlen(string) )


struct _WSMngSearchData;
typedef struct _WSMngSearchData WSMngSearchData;

/**
 * Creates new instance of WSMngSearchData structure
 *@return new instance of WSMngSearchData structure
 */
WSMngSearchData* ws_manager_create();

/** 
* \brief Function used to initialize D-BUS
*
*@param data structure stores variables which are need to comunicate by D-BUS.
*/
void ws_mng_init_dbus (WSMngSearchData *data);

/**
* \brief Function used to initialize manager
*
* Fuction loads from modules pointers to functions used to service searching 
in dictionaries and stores them in WSMngSearchData structure
*@param data pointer to structure WSMngSearchData which stores variables needed 
to service manager
*/
void ws_mng_init (WSMngSearchData *data);

/**
* \brief Function used to close dictionaries and modules
*
* @param data structure holds pointers to data which need to be freed 
from memory
*/
void ws_mng_close (WSMngSearchData *data);

/**
 * Function used to run and initialize main loop.
 *@param serach_data structure which contains all data of program
 *@return TRUE if successfully run main loop, FALSE otherwise 
 */
gboolean ws_mng_start_main_loop(WSMngSearchData* serach_data);

struct _WSMngSearchData
{
        GArray* dict; ///< pointer to GArray structure which stores pointers to fuction used to service dictionaries 
        GArray* modules; ///< pointer to GArray structure which stores pointers to fuction used to service modules loading
        gchar* word; ///< word which will be used to search a dictionaries
        pthread_t p_thread; ///< a thread in which we serching word list
	pthread_t p_thread_trans; ///< a thread in which are searching for translation of word
        WSDBusData* dbus_data; ///< a pointer DBUS data
        GMainLoop* loop; ///< pointer to main loop
        //GModule* library; ///< library used to load modules
        GArray* word_list; ///< searched word list
        gchar* trans; ///< searched translation
        gchar* last_search; ///< last searched word
        gboolean search_in_history; ///< flag signalig when search in hisory is commieted
        GArray* libraries; ///<tabele used for storing modules libraries
	Engine* bookmark; ///<bookmark engine
	/* added by Dariusz Wiechecki
	 * we are moving to GLib threads */
	GStaticMutex* action_working;
	GStaticMutex* thread_creation;

	GStaticRecMutex* action_stop;

	GArray* library_path;///<path to library
	gboolean bookmark_mode;
	GModule* bookmark_library;
};



#ifdef __cplusplus
}
#endif

#endif

