/*******************************************************************************
This file is part of WhiteStork.

WhiteStork is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

WhiteStork is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License 
along with WhiteStork; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Copyright 2006-2007 ComArch S.A.
*******************************************************************************/
#ifndef _WS_MANAGER_SEARCHING_THREADS
#define _WS_MANAGER_SEARCHING_THREADS

#include <ws_mng_threads_utils.h>
#include <dictionary_engine.h>
#include <ws_manager.h>

#ifdef __cplusplus
	extern "C" {
#endif



/** \brief Function executed by thread - search for word list
 *
 * @param search_data - pointer to search data structure
 * @return - result of thread - always NULL
 */
gpointer     ws_mng_search_word          (gpointer search_data);

/**
 * \brief Used to return results of found words from threads 
 *
 * @param list word list found in dictionaries
 * @param pattern a word which is being search for in dictionaries
 * @param user_data data passed to function
 * @param error engine status information
 */
void         ws_mng_on_found_word        (GArray* list,
                                          gchar* pattern,
                                          gpointer user_data,
                                          EngineStatus error);

/** \brief Function executed by thread - search for particular word
 *
 * @param search_data - pointer to search data structure
 * @return - result of thread - always NULL
 */
gpointer     ws_mng_search_translation   (gpointer search_data);

/**
 * \brief Used to return results of transaltion from threads
 *
 *  @param translation translation of word found in dictionaries
 * @param pattern a word which is being serch for in dictionaries
 * @param user_data data passed to function
 * @param error engine status information
 */
void         ws_mng_on_found_translation (gchar* translation,
                                          gchar* pattern,
                                          gpointer user_data,
                                          EngineStatus error);

/**
 * \brief Function used to compare string. Used in sorting GArray object
 *
 * @param a first argument to compere
 * @param b second argument to compere
 * @return result of compare <0 if the second is greater than first 0 if the 
 *       strings are the same >0 if the first string is greater than second  
 */
gint         ws_mng_compare_strings (gconstpointer a, gconstpointer b);

void         ws_remove_multiple_words(WSMngSearchAtom* user_data);

#ifdef __cplusplus
	}
#endif
#endif
