#include <untar.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <bzlib.h>
#include <sys/stat.h>
//#include <glib-2.0/glib.h>

#define BLOCK_SIZE 512
#define BUFFER_SIZE BLOCK_SIZE*32
#define ASCII_NR(NR) (NR + 0x30)

typedef struct _TarHeader
{
	gchar name[100];
	gchar mode[8];
	gchar uid[8];
	gchar gid[8];
	gchar size[12];
	gchar mtime[12];
	gchar chksum[8];
	gchar typeflag;
	gchar linkname[100];
	gchar magic[6];
	gchar version[2];
	gchar uname[32];
	gchar gname[32];
	gchar devmajor[8];
	gchar devminor[8];
	gchar prefix[155];
	gchar padding[12];
	gchar *gnu_longname;
	gchar *gnu_longlink;
} TarHeader;

typedef enum _RecordType
{
	File = ASCII_NR (0),
	ARCHLINK = ASCII_NR (1),
	SYMLINK = ASCII_NR (2),
	CHARDEV = ASCII_NR (3),
	BLOCKDEV = ASCII_NR (4),
	Dir = ASCII_NR (5),
	FIFO = ASCII_NR (6),
	RESERVED = ASCII_NR (7)
} RecordType;

static gint unpack_file_contents(BZFILE *bz2_file, 
				TarHeader *header,
				gchar *out_name);


static gint unpack_file_contents(BZFILE *bz2_file, 
				TarHeader *header,
				gchar *out_name)
{
	gchar* buffer;
	guint i;
	FILE *file_out;
	gulong length;
  	guint result;
	gint bzerror;
	
	file_out = fopen(out_name, "w");
	
	if (file_out == NULL) return -1;
	
	sscanf(header->size, "%12lo", &length);
	g_debug("File size: %ld\n", length);
	
	for (i=0; i < (length / BLOCK_SIZE); ++i)
	{
		buffer = (gchar*) g_try_malloc (BLOCK_SIZE * sizeof(gchar));
		
		result = BZ2_bzRead ( &bzerror, bz2_file, buffer, BLOCK_SIZE *
				sizeof (gchar));
		
		fwrite (buffer, BLOCK_SIZE * sizeof(gchar), 1, file_out);
		g_free (buffer);
	};
		
	i = length % BLOCK_SIZE;
				
	if (i != 0)
	{
		buffer = (gchar*) g_try_malloc (BLOCK_SIZE * sizeof(gchar));
		
		if (buffer == NULL)
		{
			g_debug("Memory not allocated");
			return -100;
		};
		
		result = BZ2_bzRead(&bzerror, bz2_file, buffer, 
				BLOCK_SIZE * sizeof(gchar));
		
		if (bzerror == BZ_OK)
		{
			fwrite (buffer, i * sizeof(gchar), 1, file_out);
		}
		
		else 
		{
			g_debug("bzerror = %d", bzerror);
			return -100;
		};
		
		g_free(buffer);
		
	};
				
	fclose(file_out);
	
	return 0;
};

gint decompress_file (gchar *in_file, gchar **out_path)
{
 	FILE *file;
	BZFILE	*bz2_file;
  	guint result;
	TarHeader* header;
	gint bzerror;
	gchar* dest_dir = NULL;
  	header = (TarHeader*) g_try_malloc (BLOCK_SIZE * sizeof(gchar));
	
	if (header == NULL) g_debug("\nCould not allocate memory\n");
	
	file = fopen (in_file, "rb");
	
	if (file != NULL)

	{
		bz2_file = BZ2_bzReadOpen (&bzerror, file, 0, 0, NULL, 0);

		if ( bzerror != BZ_OK ) 
		{
			BZ2_bzReadClose ( &bzerror, bz2_file );
			g_debug("There was an error while reading the compressed file\n");
		}
		
		bzerror = BZ_OK;
		while (1) 
		{
			
			result = BZ2_bzRead ( &bzerror, bz2_file, header, 
					       BLOCK_SIZE * sizeof (char));
			if ( bzerror == BZ_OK )
			{
				if (strlen (header->name) == 0)
				{
					g_debug("\nFilename length is 0, exitting\n");
					break;
				};
				
				g_debug("Name: %s\n", header->name);
				g_debug("Prefix: %s\n", header->prefix);
				

				gchar *temp = g_strconcat (*out_path,
							header->name, NULL);
				g_debug("Temp path %s\n", temp);
				switch (header->typeflag)
				{
					case File:
						g_debug("File\n");
						unpack_file_contents (bz2_file, 
							header, temp);
						break;
						
					case Dir:
						
						g_debug("Dir %s\n", temp);
						if (mkdir (temp, S_IRUSR|
							S_IWUSR|S_IXUSR) != 0)
						{
							g_debug("Couldn't create directory %s", 
							       temp);
							g_free(header);
							BZ2_bzReadClose(&bzerror,
									bz2_file);
							fclose(file);
							return -100;
						};
						break;
						
					default:
						g_debug("Only files and dirs can be unpacked");
				}
				dest_dir = g_strdup(temp);
				g_debug("dest_dir %s \n", dest_dir);
				g_free (temp);
				
			}
				
			else if ( bzerror != BZ_STREAM_END )
			{
				g_debug("\nBZ2 READ_CLOSE(stream end) %d\n", bzerror);
				BZ2_bzReadClose ( &bzerror, bz2_file );
				break;
			}
			
			else 
			{
				g_debug("\nExitting, error nr %d\n", bzerror);
				BZ2_bzReadClose ( &bzerror, bz2_file );
				break;
			};
			
		};
		
	}
	else 
	{
		g_debug("There was an error while trying to read the archive file");
		g_free(header); header = NULL;
		fclose(file);
		return -100;
	};
	if (dest_dir != NULL)
	{
		g_free(*out_path); *out_path = NULL;
		*out_path = g_strdup(dest_dir);
		g_free(dest_dir); dest_dir = NULL;
	}
	g_free(header); header = NULL;
	fclose(file);
	return 100;
	
	
};
