/*******************************************************************************
This file is part of WhiteStork.

WhiteStork is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

WhiteStork is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License 
along with WhiteStork; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Copyright 2006 ComArch S.A.
*******************************************************************************/
#include <ws_mng_bookmarks_utils.h>

/**
 *Function used to add bookmarks
 *@param error error messages
 *@param param word and translation of word to adding
 *@param user_data structure holding nedded parameters
 */
void ws_mng_add_bookmark(GError *error, GArray* param, gpointer user_data)
{
	g_debug("->%s", __FUNCTION__);
	
	WSMngSearchData* data = (WSMngSearchData *) user_data;
        osso_rpc_t* osso_data = NULL;
	osso_rpc_t* osso_data_trans = NULL;
        gchar* word = NULL;
	gchar* translation = NULL;
        //get the word passed by dbus 
        if (data->bookmark != NULL)
	{
		//g_debug("adding bookmarks...%p locataion", data->bookmark);
		osso_data = &g_array_index (param, osso_rpc_t, 0); 
        	word = g_strdup(osso_data->value.s);
		osso_data_trans = &g_array_index (param, osso_rpc_t, 1); 
		translation = g_strdup(osso_data_trans->value.s);
		//do not free translation and word
		gboolean is_added = dict_eng_add_word(data->bookmark, 
							word, translation);
		if (is_added == TRUE) 
		{
			ws_dbus_notify(data->dbus_data, 
				       WS_DBUS_BOOKMARKS_ADDED_OK);
		}
		else 
		{
			ws_dbus_notify(data->dbus_data, 
				       WS_DBUS_BOOKMARKS_ADDED_FAIL);
		}
		osso_rpc_free_val(osso_data);
		g_free(translation);
		g_free(word);
	}
	else
	{
		ws_dbus_notify(data->dbus_data, 
				       WS_DBUS_BOOKMARKS_ADDED_FAIL);
		g_debug("-> %s - there is no bookmark engine!\n", __FUNCTION__);
	}
	g_debug("<-%s", __FUNCTION__);
}

/**
 *Function used to remove bookmarks 
 *@param error error messages
 *@param param word and translation of word to remove
 *@param user_data structure holding nedded parameters
 */
void ws_mng_remove_bookmark(GError *error, GArray* param, gpointer user_data)
{
	g_debug("->%s", __FUNCTION__);

	WSMngSearchData* data = (WSMngSearchData *) user_data;
        osso_rpc_t* osso_data = NULL;
        gchar* word = NULL;
	//get the word passed by dbus 
        if (data->bookmark != NULL)
	{
		g_debug("removing bookmarks...");
		osso_data = &g_array_index(param, osso_rpc_t, 0); 
        	word = g_strdup(osso_data->value.s);

		gboolean is_remove  = dict_eng_remove_word(data->bookmark, word);
		if (is_remove == TRUE) 
		{
			ws_dbus_notify(data->dbus_data, 
				       WS_DBUS_BOOKMARKS_REMOVED_OK);
		}
		else 
		{
			ws_dbus_notify(data->dbus_data, 
				       WS_DBUS_BOOKMARKS_REMOVED_FAIL);
		}
		
		osso_rpc_free_val(osso_data);
	}
	else
	{	
		ws_dbus_notify(data->dbus_data, 
				       WS_DBUS_BOOKMARKS_ADDED_FAIL);
		g_debug("-> %s - there is no bookmark engine!\n", __FUNCTION__);
	}
	g_debug("<-%s", __FUNCTION__);
}


/**
 *Function used to load bookmark module 
 *@param data structure which holds the loaded module
 */
void ws_mng_load_bookmark(WSMngSearchData* data)
{
	g_debug("->%s", __FUNCTION__);
	guint i = 0;
	data->bookmark = NULL;
	gchar* current_directory = ws_mng_get_boomark_location();
	if (current_directory != NULL)
	{
	#ifdef SQLITE
	if (g_file_test(LIBRARY, G_FILE_TEST_EXISTS))
	{
	#endif
		for (i=0; i<data->modules->len; i++)
		{
			if (dict_eng_module_check(
				g_array_index(data->modules, EngineModule, i),
                                                   current_directory) == TRUE)
			{
				data->bookmark = 
					dict_eng_module_create_ext(
                                                g_array_index(data->modules,
                                                              EngineModule,
                                                              i),
                                               current_directory,
                                               ENGINE_CREATE,
                                               ws_mng_progress_bar,
                                               data,
                                               0.02);
				dict_eng_set_callback(data->bookmark,
                                                      ENGINE_WORD_LIST_SIGNAL,
                                                      ws_mng_on_found_word,
                                                      data);
                                //set callback for return translation
                                dict_eng_set_callback(data->bookmark,
                                                 ENGINE_WORD_TRANSLATION_SIGNAL,
                                                 ws_mng_on_found_translation,
                                                 data);
				
				break;
			}
		}
		
	#ifdef SQLITE
	}
	#endif
	g_free(current_directory);
	}
	g_debug("<-%s", __FUNCTION__);
}
