/*******************************************************************************
This file is part of WhiteStork.

WhiteStork is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

WhiteStork is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License 
along with WhiteStork; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Copyright 2006 ComArch S.A.
*******************************************************************************/
#include <ws_mng_dictionary_utils.h>

void ws_mng_extract_dictionary(GError *error,
                               GArray* param,
                               gpointer user_data)
{
	g_debug("->%s", __FUNCTION__);
	WSMngSearchData *data = (WSMngSearchData *) user_data;
        osso_rpc_t *osso_data;
        //get the data sended by dbus
        osso_data = &g_array_index (param, osso_rpc_t, 0); 
        //get the path sended by dbus
        gchar* path = g_strdup(osso_data->value.s);
	gint result = -1;
	
	gchar* dest = NULL;
	if (path != NULL)
	{
		gint count = 0;
		gchar** tmp = g_strsplit(path,
                		        "/",
                                        0);
		while(tmp[count++] != NULL);
		dest = g_strndup(path, g_strlen(path)- g_strlen(tmp[count -2]));
		g_strfreev(tmp);
	}
	if (dest != NULL)
	{	ws_dbus_notify(data->dbus_data, WS_DBUS_EXTRACT_FILE);
        	result = decompress_file(path, &dest);
		ws_dbus_notify(data->dbus_data, WS_DBUS_EXTRACT_FILE_FINISHED);
	}
	
	if (result > 0) 
	{
		ws_dbus_server_return_extracted_dict(data->dbus_data, dest);
	}
	else 
	{	
		dest = "";
		ws_dbus_server_return_extracted_dict(data->dbus_data, dest);
	}
	
	g_free(path);
	g_debug("<-%s", __FUNCTION__);
}


/**
 *Function used to load dictionaries engines
 *@param dict_directory path to dictionaries 
 *@param data structure which contains all data of program
*/
void ws_mng_load_dict(GArray* dict_directory, WSMngSearchData* data)
{

        gint i = 0;
        gint j = 0;
        Engine* xdxf = NULL;
        g_debug("->%s", __FUNCTION__);
        for (i =0; i<dict_directory->len; i++)
        {
                gchar* current_directory = strdup(g_array_index(dict_directory,
                                                                gchar*,
                                                                i)
                                                 );            
                for (j=0; j<data->modules->len; j++)
                {
			if (dict_eng_module_check(g_array_index(data->modules,
                                                                 EngineModule,
                                                                 j),
                                                   current_directory) == TRUE)
                        {               
                               if(ws_mng_if_optimized(current_directory))
                               {
                                        xdxf = dict_eng_module_create_ext(
                                                g_array_index(data->modules,
                                                              EngineModule,
                                                              j),
                                               current_directory,
                                               ENGINE_CREATE,
                                               ws_mng_progress_bar,
                                               data,
                                               0.02); //create engine module 
                               }
                               else
                               {
                                        xdxf = dict_eng_module_create_ext(
                                                g_array_index(data->modules,
                                                             EngineModule,
                                                             j),
                                               current_directory,
                                               ENGINE_NO, 
					       ws_mng_progress_bar,
                                               data,
                                               0.02); //create engine module 
                               }
                                //set callback for return words list function
                                dict_eng_set_callback(xdxf,
                                                      ENGINE_WORD_LIST_SIGNAL,
                                                      ws_mng_on_found_word,
                                                      data);
                                //set callback for return translation
                                dict_eng_set_callback(xdxf,
                                                 ENGINE_WORD_TRANSLATION_SIGNAL,
                                                 ws_mng_on_found_translation,
                                                 data);
                                //adding newly created engine to Garray
                                g_array_append_val (data->dict, xdxf);
                                g_free(current_directory);
                        }
                }
        }
        g_debug("<-%s", __FUNCTION__);
}


