/*******************************************************************************
This file is part of WhiteStork.

WhiteStork is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

WhiteStork is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License 
along with WhiteStork; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Copyright 2006 ComArch S.A.
*******************************************************************************/
#include <ws_mng_gconf_utils.h>


/**
 *Function used to get the dictionaries location read fron GConf
 *@return path to dictionaries location
 */ 
GArray* ws_mng_read_gconf()
{
        //g_type_init();
        g_debug("--%s", __FUNCTION__);
        GConfClient* client = NULL;
        GArray* path_to_dir = g_array_new(TRUE, TRUE, sizeof(gchar *));
        gchar* path = NULL;
        //gchar* key_value = NULL;
        gboolean key_active = FALSE;
        gint i = 0;
        
        client = gconf_client_get_default();
        GSList* list = gconf_client_all_dirs(client, GCONF_KEY, NULL);
        for (i=0; i<g_slist_length(list); i++)
        {
                //path = g_strdup((gchar *) g_slist_nth_data(list, i));
                path = (gchar* ) g_slist_nth_data(list, i);
                gchar* new_path = g_strconcat(path, "/path", NULL);
                gchar* new_acitve = g_strconcat(path, "/active", NULL);
                key_active = gconf_client_get_bool(client, new_acitve, NULL);
                if (key_active == TRUE)
                {
                        gchar* key_value = gconf_client_get_string(client,
                                                           	new_path,
                                                          	NULL);
			if (key_value != NULL){
				g_array_append_val(path_to_dir, key_value);
			}
                }
                g_free(new_path);
                g_free(new_acitve);
                
        }
        for (i=0; i<g_slist_length(list); i++)
        {
                g_free(g_slist_nth_data(list, i));
        }
        g_slist_free(list);
        g_object_unref (client);
        return path_to_dir;
}


/**
 *Function used to get plugins location read from GConf
 *@return libraries location
 */
GArray* ws_mng_get_engines_location()
{

        //g_type_init();
        g_debug("->%s", __FUNCTION__);
        GConfClient* client = NULL;
        GArray* path_to_dir = g_array_new(TRUE, TRUE, sizeof(gchar *));
        gchar* path = NULL;
        gint i = 0;
        client = gconf_client_get_default();
        GSList* list = gconf_client_all_dirs(client, GCONF_LIB_PATH, NULL);
        for (i=0; i<g_slist_length(list); i++)
        {
                //path = g_strdup((gchar *) g_slist_nth_data(list, i));
                path = (gchar* ) g_slist_nth_data(list, i);
                gchar* new_path = g_strconcat(path, "/path",NULL);
                gchar* key_value = gconf_client_get_string(client,
                                                           new_path,
                                                           NULL);
		gchar** tmp = g_strsplit(key_value,
                                         "/",
                                         -1);
		guint i = 0;
		gboolean bookmark_engin = FALSE;
		while(tmp[i] != NULL)
		{
			g_debug("Comapred string %s", tmp[i]);
			if (g_ascii_strcasecmp(tmp[i], "ws_bookmark.so") == 0)
			{
				bookmark_engin = TRUE;
				break;
			}
			i++;
		}
		g_strfreev(tmp);
		#ifdef SQLITE
		if (bookmark_engin)
		{
			gboolean result = g_file_test(
						LIBRARY,
						G_FILE_TEST_EXISTS);
			if (result == FALSE)
			{
				g_debug("sqlite not installed !");
				g_free(new_path);
				continue;
			}
		}
		#endif
		
		if (key_value != NULL)
		{
			g_debug("Added library path %s", key_value);
			g_array_append_val(path_to_dir, key_value);
		}
                g_free(new_path);
        }
        for (i=0; i<g_slist_length(list); i++)
        {
                g_free(g_slist_nth_data(list, i));
        }
        g_slist_free(list);
        g_object_unref (client);

        return path_to_dir;
}


/**
 *Function used to get bookmark library location
 *@return path to bookmark location
 */
gchar* ws_mng_get_boomark_location()
{
	g_debug("->%s", __FUNCTION__);
	//g_type_init();
        GConfClient* client = NULL;
        gchar* path = NULL;
        guint i = 0;
        client = gconf_client_get_default();
        GSList* list = gconf_client_all_dirs(client, GCONF_KEY, NULL);
        for (i=0; i<g_slist_length(list); i++)
        {
                //path = g_strdup((gchar *) g_slist_nth_data(list, i));
                path = (gchar* ) g_slist_nth_data(list, i);
                gchar* name = g_strconcat(path, "/name",NULL);
		gchar* new_path = g_strconcat(path, "/path",NULL);
		gchar* key_value = gconf_client_get_string(
							client,
                                                        name,
                                                        NULL);
		//g_debug("Try this: %s and the path is %s", key_value, path);
		if (key_value != NULL)
		{
			if (g_ascii_strcasecmp(key_value, "Bookmarks") == 0)
			{
				gchar* key_value_path = gconf_client_get_string(
									client,
                                        	                	new_path,
                                                	        	NULL);
				
				if (key_value_path != NULL)
				{
					g_free(key_value);
					key_value = NULL;
					g_free(new_path);
					new_path = NULL;
					g_free(name); name = NULL;	
					for (i=0; i<g_slist_length(list); i++)
        				{
                				g_free(
						g_slist_nth_data(list, i));
        				}
        				g_slist_free(list);
        				g_object_unref(client);
					return key_value_path;
				}
			}
		}
		g_free(name); name = NULL;
		g_free(key_value);
		key_value = NULL;
		g_free(new_path);
		new_path = NULL;
        }
        
	g_debug("<-%s", __FUNCTION__);
        return NULL;
}

/**
 *Function used to check if optimized flag in GConf is set
 *@param dict path to dictionary file
 *@return TRUE if dictionary is optimize, FALSE otherwise
 */
gboolean ws_mng_if_optimized(gchar* dict)
{
	g_debug("->%s", __FUNCTION__);

	GConfClient* client;
	gchar* path = NULL;
        gboolean key_optimized = FALSE;
	gboolean key_found = FALSE;
        gint i = 0;

        client = gconf_client_get_default();
        GSList* list = gconf_client_all_dirs(client, GCONF_KEY, NULL);
        for (i=0; i<g_slist_length(list); i++)
        {
                path = (gchar *) g_slist_nth_data(list, i);
                gchar* new_path = g_strconcat(path, "/path",NULL);
                gchar* new_optimized = g_strconcat(path, "/optimized",NULL);
                gchar* key_value = gconf_client_get_string(client,
                                                           new_path,
                                                           NULL
                                                          );
		if (g_ascii_strcasecmp(key_value, dict)==0)
		{
			key_optimized = gconf_client_get_bool(client,
			                                      new_optimized,
			                                      NULL
		                                              );
			key_found = TRUE;
                }
                g_free(new_path);
                g_free(new_optimized);
                g_free(key_value);
		if ( TRUE == key_found )
		{
			break;
		}
        }
        for (i=0; i<g_slist_length(list); i++)
        {
                g_free(g_slist_nth_data(list, i));
        }
        g_slist_free(list);
        g_object_unref (client);

	g_debug("<-%s", __FUNCTION__);
	return key_optimized;
}


