/*******************************************************************************
This file is part of WhiteStork.

WhiteStork is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

WhiteStork is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License 
along with WhiteStork; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Copyright 2006 ComArch S.A.
*******************************************************************************/
#ifndef _DICTIONARY_ENGINE_XDXF
#define _DICTIONARY_ENGINE_XDXF

#ifdef __cplusplus
        extern "C" {
#endif

//______________________________________________________________________________
// *****************************************************************************
//************************************************************* HEADERS SECTION:
//------------------------------------------------------------------------------
// headers with unix types/functions - onl for timers
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <fcntl.h>     
#include <unistd.h>   
//------------------------------------------------------------------------------
// header with GLIB definitions/functions/types
#include <glib.h>
//------------------------------------------------------------------------------
// header with gnome-vfs - recommended I/O API for maemo
#include <libgnomevfs/gnome-vfs.h>
//------------------------------------------------------------------------------
// header with expat - XML Parser API
#include <expat.h>
//------------------------------------------------------------------------------
// header wit engine API
#include "dictionary_engine.h"
//------------------------------------------------------------------------------


//______________________________________________________________________________
// *****************************************************************************
//********************************************************* DEFINITIONS SECTION:
//------------------------------------------------------------------------------
// definitions for timer function - flag telling if we want to start or stop
// timing
#define TIMER_START     TRUE
#define TIMER_STOP      FALSE
//------------------------------------------------------------------------------
// definitions of version and format which engine handles
#define DIC_ENG_VERSION "0.1b"
#define DIC_ENG_FORMAT  "XDXF"
//------------------------------------------------------------------------------
// defines lenght of buffers for particular functions in engine which needs
// big buffers
#define DICT_CACHEING_BUFF_SIZE                 16*1024
#define DICT_SEARCHING_WORD_LIST_BUFF_SIZE      16*1024
#define DICT_SEARCHING_WORD_TRAN_BUFF_SIZE      16*1024
//------------------------------------------------------------------------------
// maximum length of word possible to find in any dictionary
#define DICT_MAX_WORD_LENGTH   512
//------------------------------------------------------------------------------
// macro for "printing" gboolean statement - "TRUE" or "FALSE"
#define PRINT_STATE(state) ( (state) ? "TRUE" : "FALSE" )
//------------------------------------------------------------------------------


//______________________________________________________________________________
// *****************************************************************************
//****************************************** DATA STRUCTURE DEFINITIONS SECTION:
//------------------------------------------------------------------------------
/** \brief Structure used while dict_eng_module_check() is working.
 */
struct _XDXFCheckingData {
        gboolean        further;
        gboolean        good;
        guint                deep;
};
typedef struct _XDXFCheckingData        XDXFCheckingData;
//------------------------------------------------------------------------------
/** \brief Structure to help parse xdxf file for searching words list.
 */
struct _XDXFWordsListData {
        gchar* last_word;
        gchar* pattern;
        guint  pattern_len;
        guint  last_word_length;
        GArray* result;
        guint  one_word;
        gboolean cont;
};
typedef struct _XDXFWordsListData       XDXFWordsListData;
//------------------------------------------------------------------------------
/** \brief Structure to help parse xdxf file for searching word's translation.
 */
struct _XDXFWordsTransData {
        gchar* last_word;
        gchar* word;
        guint  word_len;
        guint  last_word_length;
        gchar* translation;
        guint  one_word;
        gboolean cont;
        gulong last_start;
        XML_Parser* parser;
        gboolean found;
        GnomeVFSHandle*        xdxf;
};
typedef struct _XDXFWordsTransData      XDXFWordsTransData;
//------------------------------------------------------------------------------
/** \brief Structure to help make optimization possible
 */
struct _XDXFCacheData {
        gchar* buffer;
        long last_start;
        long last_stop;
        long last_length;
        GnomeVFSHandle* cache;
        XML_Parser parser;
        int state;
        long buffer_length;
};
typedef struct _XDXFCacheData           XDXFCacheData;
//------------------------------------------------------------------------------
/** \brief Internal data structure for representing part of file.
 */
struct _FilePart {
        guint offset;
        guint length;
};
typedef struct _FilePart        FilePart;
//------------------------------------------------------------------------------
/** \brief Internal data structure of XDXF Engine.
 */
struct _XDXFData {
        GnomeVFSHandle*         xdxf;
        GnomeVFSHandle*         cache;
        gchar*                  dict_path;
        EngineStatus            last_error;
        gboolean                auto_free;

        cb_progress             cb_progress_caching;
        gpointer                cb_progress_caching_data;
        gdouble                 cb_progress_caching_seed;

        cb_progress             cb_progress_word_list;
        gpointer                cb_progress_word_list_data;
        gdouble                 cb_progress_word_list_seed;

        cb_progress             cb_progress_word_trans;
        gpointer                cb_progress_word_trans_data;
        gdouble                 cb_progress_word_trans_seed;

        cb_word_list            cb_search_word_list;
        gpointer                cb_search_word_list_data;

        cb_word_translation     cb_search_word_trans;
        gpointer                cb_search_word_trans_data;      
};
typedef struct _XDXFData        XDXFData;
//------------------------------------------------------------------------------


//______________________________________________________________________________
// *****************************************************************************
//*************************************************** PARSING FUNCTIONS SECTION:
//------------------------------------------------------------------------------
// while is_xdxf_file() is working
static void     is_xdxf_file_start(void *data,
                                   const char *el,
                                   const char **attr);
static void     is_xdxf_file_end(void *data,  const char *el);
//------------------------------------------------------------------------------                                                 
// while dict_eng_search_word_list() is working
static void     search_word_list_start(void *data,
                                       const char *el,
                                       const char **attr); 
static void     search_word_list_end(void *data, const char *el);
static void     search_word_list_text(void *data, const XML_Char *txt, int len);
//------------------------------------------------------------------------------
// // while dict_eng_search_word_translation() is working
static void     search_word_trans_start(void *data,
                                        const char *el,
                                        const char **attr);
static void     search_word_trans_end(void *data, const char *el);                             
static void     search_word_trans_text(void *data,
                                       const XML_Char *txt,
                                       int len);
//------------------------------------------------------------------------------


//______________________________________________________________________________
// *****************************************************************************
//************************************************ ADDITIONAL FUNCTIONS SECTION:
//------------------------------------------------------------------------------
// returning concrete part of file
static gchar*           read_file_part(FilePart* part, GnomeVFSHandle* file);
//------------------------------------------------------------------------------
// convert string to proper path name (no filename, no "/" at the ned, file 
// exist)
static gchar*           string_to_path(gchar** string);
//------------------------------------------------------------------------------
// tells if file is in XDXF format (file should exist)
static gboolean         is_xdxf_file(gchar* file); 
//------------------------------------------------------------------------------
// start/stop timers -  returnet -1.0 if we start or seconds passed from start 
// if we want to stop timer
//static double           timer(gboolean start, gchar* message);
//------------------------------------------------------------------------------
// return size of files
static guint64          get_file_size(GnomeVFSHandle* file);
//------------------------------------------------------------------------------
// return how many records (from cache file) are in current buffer a with length
// length.
static guint            get_max_length(gchar* a, guint length);
//------------------------------------------------------------------------------
// return translation of word using cache file
static gchar* word_translation_cache(XDXFData* data, gchar* word);
//------------------------------------------------------------------------------
// return translation of word but using only xdxf dictionary file
static gchar* word_translation_xdxf(XDXFData* data, gchar* word);
//------------------------------------------------------------------------------
// return translation of word using cache file
static void word_list_cache(XDXFData* data,
                            gchar* pattern,
                            GArray* result,
                            gpointer cb_data);
//------------------------------------------------------------------------------
// return translation of word but using only xdxf dictionary file
static void word_list_xdxf(XDXFData* data,
                           gchar* pattern,
                           GArray* result,
                           gpointer cb_data);
//------------------------------------------------------------------------------



//______________________________________________________________________________
// *****************************************************************************
//****************************************************** MAIN FUNCTIONS SECTION:
//------------------------------------------------------------------------------
// implementation of dict_eng_module_check(module,location) function
gboolean        xdxf_engine_check(gchar* location);
//------------------------------------------------------------------------------
// implementation of dict_eng_module_get_description(module) function
gchar*          xdxf_engine_description();
//------------------------------------------------------------------------------
// implementation of dict_eng_module_get_format(module) function
gchar*          xdxf_engine_format();
//------------------------------------------------------------------------------
// implementation of dict_eng_module_get_version(module) function
gchar*          xdxf_engine_version();
//------------------------------------------------------------------------------
// implementation of dict_eng_module_create(module,location,flags) and
// dict_eng_module_create_ext(module,location,flags) functions
Engine*         xdxf_engine_create(gchar* location, 
                              EngineOptimizationFlag flags,
                              cb_progress progress_handler,
                              gpointer progress_data,
                              gdouble seed);
//------------------------------------------------------------------------------
// implementation of dict_eng_destroy(engine) function
void            xdxf_engine_close(Engine* engine);
//------------------------------------------------------------------------------
// implementation of dict_eng_get_location(engine) function
gchar*          xdxf_engine_location(Engine* engine);
//------------------------------------------------------------------------------
// implementation of dict_eng_optimize(engine) function
void            xdxf_engine_optimize(Engine* engine);
//------------------------------------------------------------------------------
// implementation of dict_eng_is_optimized( engine ) function
gboolean        xdxf_engine_is_optimized(Engine* engine);
//------------------------------------------------------------------------------
// implementation of dict_eng_set_auto_free(engine, state) function
void            xdxf_engine_set_auto_free(Engine* engine, gboolean state);
//------------------------------------------------------------------------------
// implementation of dict_eng_set_callback(engine,signal,c_handler,data) 
// function
gpointer        xdxf_engine_set_callbacks(Engine* engine,
                                     gchar* event,
                                     gpointer c_handler,
                                     gpointer user_data);
//------------------------------------------------------------------------------
// implementation of dict_eng_set_progress_seed(engine, signal, val) function
void            xdxf_engine_set_progress_seed(Engine* engine,
                                         gchar* signal,
                                         gdouble seed);
//------------------------------------------------------------------------------
// implementation ofdict_eng_search_word_list(engine,pattern) function
void            xdxf_engine_search_word_list(Engine* engine,
                                             gchar* pattern,
                                             gpointer data);
//------------------------------------------------------------------------------
// implementation of dict_eng_search_word_translation(engine,word) function
void            xdxf_engine_search_word_translation(Engine* engine,
                                                    gchar* word,
                                                    gpointer data);
//------------------------------------------------------------------------------
// implementation of dict_eng_search_word_translation_extended(engine,word)
// function
void            xdxf_engine_search_word_translation_extended(Engine* engine,
                                                        gchar* word);
//------------------------------------------------------------------------------
// implementation of dict_eng_get_last_state(engine) function
EngineStatus    xdxf_engine_error(Engine* engine);
//------------------------------------------------------------------------------
// implementation of dict_eng_state_message(error) function
gchar*          xdxf_engine_error_message(EngineStatus error);
//------------------------------------------------------------------------------
// implementation of engine_global_functions(void) function
// API 2.0
gboolean 	xdxf_engine_add_word(Engine* engine,
                         	gchar*  word,
                         	gchar*  translation);

gboolean 	xdxf_engine_remove_word(Engine* engine, gchar*  word);

EngineModule    engine_global_functions();

#ifdef __cplusplus
}
#endif
#endif
